/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: P. Dechamboux, Y. Devaux, F. Dhumeaux.
 *
 */
package org.objectweb.speedo.api;

import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.speedo.api.SpeedoProxy;
import org.objectweb.perseus.persistence.api.TransactionalPersistenceManager;

import javax.jdo.PersistenceManager;

/**
 * This interfaces defined a manager of proxies (@see SpeedoProxy). This concept
 * is based on the javax.jdo.PersistenceManager interface.
 * A proxy manager must be able:<UL>
 * <LI>to create identifiers,</LI>
 * <LI>to active or descative a proxy,</LI>
 * <LI>to retrieves the PClassMapping (persistent object familly manager in
 * Jorm)</LI>
 * </UL>
 *
 * @author S.Chassande-Barrioz
 */
public interface ProxyManager extends PersistenceManager {

    /**
     * Sets the reference between two JDO instances
     *
     * @param pc the reference object whose proxy infrastructure will be build
     * @param father the owner
     * @param field the name of the field
     */
    PName createFieldOfProxy(SpeedoProxy pc, SpeedoProxy father, String field) throws Exception;

    /**
     * Removes the proxy infrastructure of a JDO instance
     *
     * @param pc the object requiring architecture deletion or its oid
     * @param retainValues true if fields have to be restored into the JDO instance
     * @return the modified non-persistant JDO instance
     */
    //SpeedoProxy desactiveProxy(SpeedoProxy pc, boolean retainValues) throws Exception;

    TransactionalPersistenceManager getTransactionalPersistenceManager();

    /**
     * Opens the persistent manager.
     * @param connectionSpec is the information to access to the datastore
     * (user, passwor, ...)
     */
    void open(Object connectionSpec);

    Object getConnectionSpec();
}
