/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: P. Dechamboux, Y. Devaux, F. Dhumeaux.
 *
 */
package org.objectweb.speedo.api;

import org.objectweb.speedo.api.SpeedoException;

import javax.jdo.PersistenceManagerFactory;
import java.util.Collection;

/**
 * This interfaces defines a ProxyManager switch. It bindes a thread to a
 * list of ProxyManager. In a same context two proxy managers cannot be managed
 * by the same PersistentManagerFactory.
 *
 * @author S.Chassande-Barrioz
 */
public interface ProxyManagerSwitch {

    /**
     * @param pmf is persistent manager factory which manages the returned
     * proxy manager.
     * @return the ProxyManager managed by the given persistence manager factory
     * and bound to current the context, or the null value if there is no
     * ProxyManager.
     */
    ProxyManager lookup(PersistenceManagerFactory pmf);

    /**
     * It assignes a ProxyManager to the current context.
     * @param pm is the ProxyManager
     * @exception org.objectweb.speedo.api.SpeedoException is thrown when a proxy manager which has the
     * same PersistentManagerFactory is already bound to the current context.
     */
    void bind(ProxyManager pm) throws SpeedoException;

    /**
     * It clears the list of ProxyManager for the current context.
     */
    void clear();

    /**
     * It clears a ProxyManager for the current context.
     */
    boolean remove(ProxyManager pm);

    /**
     * @return all ProxyManager instances bound with the current context
     */
    Collection entries();
}
