/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: P. Dechamboux, Y. Devaux, F. Dhumeaux.
 *
 */

package org.objectweb.speedo.api;

import org.objectweb.perseus.cache.replacement.api.ReplaceableCacheEntry;
import org.objectweb.jorm.api.PAccessor;

/**
 * This interface a state of an user instance.
 *
 * @author Y. Devaux
 */
public interface SpeedoAccessor extends PAccessor, ReplaceableCacheEntry {

    /**
     * Changes the status of this object
     *
     * @see org.objectweb.speedo.api.LifeCycle
     * @param action the action that may change the status
     */
    void jdoChangeStatus(byte action);

    /**
     * Gets the current status of this object
     *
     * @see org.objectweb.speedo.api.LifeCycle
     * @return the current state in the life cycle
     */
    byte jdoGetStatus();

    /**
     * Forces the new value of the status
     *
     * @see org.objectweb.speedo.api.LifeCycle
     * @param newValue the new status of this object
     */
    void jdoSetStatus(byte newValue);

    /**
     * @return The SpeedoProxy attached to this state representation.
     */
    SpeedoProxy jdoGetSpeedoProxy();

    /**
     * Assignes the SpeedoProxy attached to this state representation.
     */
    void jdoSetSpeedoProxy(SpeedoProxy sp);

    /**
     * @return The context associated to the accessor.
     */
    Transaction jdoGetTransaction();

    /**
     * Assignes the context with which the accessor is bound.
     * @param tx is the context
     */
    void jdoSetTransaction(Transaction tx);
}

