/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.api;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.naming.api.PName;

/**
 * This interfaces represents a SpeedoProxy for a generic class (Collectio, Set,
 * Map, ...)
 * It permits to specify the mapper throught the generic class is persistent.
 * The Mapper name permits to know the type of GenClassMapping and
 * GenClassBinding (rdb, fos, ...).
 *
 * The PType of the generic class permit to known which is GenClassMapping
 * instance to use.
 *
 * @author S.Chassande-Barrioz
 */
public interface SpeedoGenClassProxy extends SpeedoProxy {

    /**
     * @param mn is the mapper name by which the instance is persistent.
     */
    void jdoSetMapperName(String mn);

    /**
     * Assignes the PType of the generic class.
     * @param ptype of the generic class
     */
    void jdoSetPType(PType ptype);

    /**
     * @return the PType of the generic class.
     */
    PType jdoGetPType();

    /**
     * @return a string which represents the fully path of the persistent field
     * of the class with wich the generic class is persistent.
     * If the linked fiekf is the 'f1' provided by the class 'com.foo.Bar'
     * then the returned string will be 'com.foo.Bar/f1'
     */
    String jdoGetGenClassId();

    /**
     * Assignes the linked field name for this generic class.
     * @param lf is tring which represents the fully path of the persistent
     * field of the class with wich the generic class is persistent.
     * If the linked fiekf is the 'f1' provided by the class 'com.foo.Bar'
     * then the expected string is 'com.foo.Bar/f1'
     */
    void jdoSetLinkedField(String lf);

    /**
     * assignes the PBinding to use to load/store the generic class
     * Call this method with null value means that the generic is no more
     * persistent.
     * @param pb is the PBinding.
     */
    void jdoSetPBinding(PBinding pb);

    /**
     * @return the pbinding associated to the generic class. The value is not
     * used if the generic class is not persistent.
     */
    PBinding jdoSetPBinding();

    /**
     * Assignes the new value of the generic class (Collection, Map or Array
     * or ...).
     */
    void setInner(Object o);
}
