/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: P. Dechamboux, Y. Devaux, F. Dhumeaux.
 *
 */
package org.objectweb.speedo.api;

import org.objectweb.perseus.cache.api.CacheEntry;
import org.objectweb.perseus.persistence.api.WorkingSet;
import org.objectweb.perseus.persistence.api.PersistenceException;

import java.util.Collection;

/**
 * This interfaces defines the transaction concept in Speedo. It is based on a
 * working set represented by the inheritance of the PersistenceManager
 * interface, and the required javax.jdo.Transaction interface.
 *
 * @author S.Chassande-Barrioz
 */
public interface Transaction extends javax.jdo.Transaction, WorkingSet {

    /**
     * It activates the working set. This is used to delimit the begining of
     * the working set.
     */
    void active() throws PersistenceException;

    /**
     * Attach an object which permits to access the data stored in a support, to
     * the working set.
     * @param dataaccess
     */
    void assignDataAccess(Object dataaccess);

    /**
     * Retrieves the data access object
     */
    Object getDataAccess();

    /**
     * Dettach the data access to the working set
     * @param dataaccess
     */
    void removeDataAccess(Object dataaccess);

    /**
     * Indicates if the transaction environnement is managed. If the 'false'
     * value is returned then that means that a user transaction matches to
     * a local transactioni managed by speedo.
     */ 
    boolean isManagedEnv();
}
