package org.objectweb.speedo.generation.enhancer;

import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoEnhancerException;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.speedo.metadata.SpeedoPackage;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.generation.lib.NamingRules;
import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Iterator;
import java.util.Collection;
import java.util.Set;
import java.util.TreeSet;
import java.io.File;

public class PersistenceAwareEnhancer extends EnhancerComponent {

    public final static String LOGGER_NAME
            = "org.objectweb.speedo.generation.enhancer.aware";

    /**
     * Indicates whether the <code>.class</code> files would be found in an
     * archive.
     */
    protected boolean isSrcJar;

    /**
     * Initializes this PersistenceAwareEnhancer
     */
    public void init() {
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
        //TODO: manage classes localized in a jar file
        isSrcJar = false;
    }

    /**
     * Loads all persistence aware classes described by the Object Model and
     * applies revelant modification to each of them.
     *
     * @exception SpeedoEnhancerException if something goes wrong
     */
    public void process() throws SpeedoException {
        if (scp.xmldescriptor == null || scp.xmldescriptor.isEmpty())
            throw new SpeedoEnhancerException("xmldescriptor not initialised");

        Set generatedClasses = computeGeneratedClasses();

        Collection xmls = scp.xmldescriptor.values();
        Iterator i = scp.awareFiles.iterator();
        while (i.hasNext()) {
            String awareFile = (String) i.next();
            String name = awareFile.replace(File.separatorChar, '.');
            name = name.substring(0, name.length() - 6); // removes the ".class"

            if (generatedClasses.contains(name)) {
                // do not enhance previously generated classes
                continue;
            }

            logger.log(
                    BasicLevel.DEBUG,
                    "Enhancing persistent aware class '" + name + "'");

            ClassReader cr = loadJavaClass(
                    isSrcJar,
                    name,
                    scp.awareFilesDir,
                    false);
            ClassWriter cw = new ClassWriter(false);

            PersistenceAwareClassModifier awarecmodifier =
                    new PersistenceAwareClassModifier(cw, xmls, logger);

            cr.accept(awarecmodifier, false);

            writeJavaClass(name, cw, scp.output);
        }
    }

    private Set computeGeneratedClasses() {
        Set result = new TreeSet();
        Collection xmls = scp.xmldescriptor.values();
        for (Iterator itDesc = xmls.iterator(); itDesc.hasNext();) {
            SpeedoXMLDescriptor desc = (SpeedoXMLDescriptor) itDesc.next();
            for (Iterator itPack = desc.jdoPackage.values().iterator(); itPack.hasNext();) {
                SpeedoPackage sp = (SpeedoPackage) itPack.next();
                for (Iterator itclass = sp.jdoClass.values().iterator(); itclass.hasNext();) {
                    SpeedoClass jdoClass = (SpeedoClass) itclass.next();
                    String name = sp.name + '.' + jdoClass.name;
                    String proxyName = NamingRules.proxyName(name);
                    String accessorName = NamingRules.accessorName(name);
                    String bindingName = sp.name + '.' + scp.mapperName + '.' + NamingRules.bindingName(jdoClass.name);
                    String mappingName = sp.name + '.' + scp.mapperName + '.' + NamingRules.mappingName(jdoClass.name);
                    String iteratorName = sp.name + '.' + scp.mapperName + '.' + jdoClass.name + "ResultIterator";
                    String fieldsName = NamingRules.fieldsName(name);
                    result.add(name);
                    result.add(proxyName);
                    result.add(accessorName);
                    result.add(bindingName);
                    result.add(mappingName);
                    result.add(iteratorName);
                    result.add(fieldsName);
                }
            }
        }
        return result;
    }
}
