/**
 * copyright area
 */
package org.objectweb.speedo.generation.generator.api;

import org.apache.velocity.app.VelocityEngine;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.generation.api.GeneratorComponent;
import org.objectweb.speedo.metadata.SpeedoClass;

import java.util.Collection;

/**
 * This interface represents a component of a generator which generates a file
 * with the Velocity tool. The use of this interface permits to reuse the
 * VelocityEngine in order have better generation performance. Indeed the
 * VelocityEngine initialization is very cheap.
 *
 * @author S.Chassande-Barrioz
 */
public interface VelocityGenerator extends GeneratorComponent {

    /**
     * It assignes the VelocityEngine instance which must be used during the
     * veleocity generatioin.
     * @param ve is the VelocityEngine instance to use.
     */
    void setVelocityEngine(VelocityEngine ve);

    /**
     * @return the VelocityEngine used during the velocity generation.
     */
    VelocityEngine getVelocityEngine();

    /**
     * @return a collection of the file names which contains macros. This file
     * name list will be given to the VelocityEngine during its initialization.
     */
    Collection getExternalsTemplate();

    /**
     * It generates the file name corresponding to the Speedo meta object given
     * in parameter.
     * @param sClass is the speedo meta object which represents a persistent class
     * @param fileName is the generated file name
     * @throws SpeedoException
     */
    void generate(SpeedoClass sClass, String fileName) throws SpeedoException;
}
