/**
 * copyright area
 */
package org.objectweb.speedo.generation.generator.lib;

import org.objectweb.speedo.generation.lib.AbstractGeneratorComponent;
import org.objectweb.speedo.generation.generator.api.VelocityGenerator;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoIdentity;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.apache.velocity.app.VelocityEngine;

import java.util.Collection;
import java.io.File;

/**
 * @author S.Chassande-Barrioz
 */
public abstract class AbstractVelocityGenerator
        extends AbstractGeneratorComponent
        implements VelocityGenerator {

    public final static String TEMPLATE_DIR
            = "org.objectweb.speedo.generation.generator";

    protected VelocityEngine ve = null;

    protected static char fs = File.separatorChar;

    protected String templateFN = null;

    /**
     * This method indicates if the type is a boolean, a char, a short, an int,
     * a long, a float, a double or a String.
     *
     * @param type the type to determine.
     * @return true if the String is one of those above.
     */
    protected static boolean isClassicalType(String type) {
        return type.equals("boolean")
                || type.equals("char")
                || type.equals("short")
                || type.equals("int")
                || type.equals("long")
                || type.equals("float")
                || type.equals("double")
                || type.equals("String")
                || type.equals("byte");
    }

    public boolean isContainerIdField(Class clazz, TypedElement te, SpeedoClass sc) {
        return sc.identityType == SpeedoIdentity.CONTAINER_ID
                && clazz
                .getNameDef(scp.projectName)
                .getNameRef().getProjection().containsValue(te.getName());
    }



    // IMPLEMENTATION OF THE VelocityGenerator INTERFACE //
    //---------------------------------------------------//

    public VelocityEngine getVelocityEngine() {
        return ve;
    }

    public void setVelocityEngine(VelocityEngine ve) {
        this.ve = ve;
    }

    public Collection getExternalsTemplate() {
        return null;
    }

    /**
     * generate the file name corresponding to the Speedo meta object given in
     * parameter.
     * @param sClass is the speedo meta object which represents a persistent class
     * @param fileName is the generated file name
     * @throws SpeedoException
     */
    public abstract void generate(SpeedoClass sClass, String fileName)
            throws SpeedoException;

    // IMPLEMENTATION OF THE GeneratorComponent INTERFACE //
    //----------------------------------------------------//

    public abstract void init() throws SpeedoException;

    public final void process() throws SpeedoException {
        throw new SpeedoException("Do not use this method");
    }
}
