/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: P. Dechamboux, Y. Devaux, F. Dhumeaux.
 *
 */

package org.objectweb.speedo.generation.jorm;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.metainfo.api.ClassMapping;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.metainfo.api.ClassProject;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoXMLError;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.generation.lib.AbstractGeneratorComponent;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.metadata.SpeedoModifier;
import org.objectweb.speedo.metadata.SpeedoPackage;
import org.objectweb.speedo.metadata.SpeedoIdentity;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.jorm.util.io.lib.DirJavaExplorer;
import org.objectweb.jorm.comp.compiler.lib.JormCompiler;
import org.objectweb.jorm.comp.compiler.lib.JormCompilerParameter;
import org.objectweb.jorm.comp.compiler.lib.JormCompilerConfigurator;
import org.objectweb.util.monolog.api.BasicLevel;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.HashSet;
import java.util.Set;
import java.util.HashMap;
import java.util.Collections;

/**
 * Generates the JORM Files corresponding to a set of Java Classes.
 * <p>See <a href="http://www.objectweb.org/jorm">http://www.objectweb.org/jorm</a>for more information.</p>
 * @author	Y. Devaux
 */
public class JormGenerator extends AbstractGeneratorComponent {

    public final static String LOGGER_NAME
            = "org.objectweb.speedo.generation.jorm";

    /**
     * Jorm classes intended for JORM generation
     */
    protected JormCompiler jormcompiler;


    // IMPLEMENTATION OF THE GeneratorComponent INTERFACE //
    //----------------------------------------------------//

    public void init() throws SpeedoException {
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
        jormcompiler = new JormCompiler();
        JormCompilerParameter jcp = jormcompiler.getCompilerParameter();
        JormCompilerConfigurator jcc = jormcompiler.getCompilerConfigurator();
        try {
            jcc.configure();
            jcp.loadConfFile(jcc.getGlobalJormcOptsFile(), jcc.knownMappers());
            jcc.setLoggerFactory(scp.loggerFactory);
            jcp.projectName = scp.projectName;
            jcp.bindingInheritance = "%p%c";
            jcp.output = scp.output;
            jcp.bindingAbstract = true;
            jcp.dtdLocations = scp.dtdLocations;
            jcp.inputFiles = scp.jorm;
            DirJavaExplorer pe = new DirJavaExplorer();
            pe.setLogger(scp.loggerFactory.getLogger("org.objectweb.jorm.io.pathexplorer"));
            pe.addPath(scp.jormclasspath);
            jcp.classpath = pe;
            jcp.subMappers = new HashMap(1);
            int idx = scp.mapperName.indexOf('.');
            if (idx == -1) {
                jcp.subMappers.put(scp.mapperName,
                        Collections.singletonList("generic"));
            } else {
                jcp.subMappers.put(
                        scp.mapperName.substring(0,idx),
                        Collections.singletonList(scp.mapperName.substring(idx + 1)));
            }
            Collection c = jcp.getSubMappers("rdb");
            logger.log(BasicLevel.DEBUG, "jormcOpts file:" + jcc.getJormcOptsFile());
            logger.log(BasicLevel.DEBUG, "rdb sub mappers: " + c);

        } catch (PException e) {
            logger.log(BasicLevel.ERROR,
                    "Impossible to configure Jorm", ExceptionHelper.getNested(e));
            throw new SpeedoException("Impossible to configure Jorm", e);
        }
    }

    /**
     * Performs the JORM objects' generation.
     */
    public void process() throws SpeedoException {
        try {
            Collection c = jormcompiler.getCompilerParameter().inputFiles;
            if (logger.isLoggable(BasicLevel.DEBUG))
                for (Iterator it = c.iterator(); it.hasNext();)
                    logger.log(BasicLevel.DEBUG, "source file: " + it.next());

            ArrayList notFoundMOs = new ArrayList();
            for (Iterator itDesc = scp.xmldescriptor.values().iterator(); itDesc.hasNext();) {
                SpeedoXMLDescriptor desc = (SpeedoXMLDescriptor) itDesc.next();
                List scs = desc.getSpeedoClasses();
                for (int i=0; i<scs.size(); i++) {
                    SpeedoClass sc = (SpeedoClass) scs.get(i);
                    String fn = sc.getJormFileName();
                    logger.log(BasicLevel.DEBUG, "looking for the file: " + fn);
                    if (!c.contains(fn)) {
                        notFoundMOs.add(sc);
                    }
                }
            }

            // if .pd files are not specified by the user then the jorm meta
            // information is generated
            jormcompiler.setupLogger();
            jormcompiler.setupMIManager();
            Collection createdMOs = null;
            if (!notFoundMOs.isEmpty()) {
                createdMOs = new JormMIBuilder(
                        jormcompiler.getMIManager(), logger)
                        .createMI(notFoundMOs, scp.projectName, scp.mapperName);
            }


            // parse the .pd files
            c = new ArrayList(jormcompiler.parseFiles(c));
            isCompatible();
            if (scp.generateNeededJormFile
                    && createdMOs != null && !createdMOs.isEmpty()) {
                jormcompiler.generateJormFiles(createdMOs);
            }
            c.addAll(createdMOs);

            if (logger.isLoggable(BasicLevel.DEBUG))
                for (Iterator it = c.iterator(); it.hasNext();) {
                    Object o = it.next();
                    if (o instanceof Class) {
                        logger.log(BasicLevel.DEBUG, "Class : "
                                + ((Class) o).getFQName());
                    } else if (o instanceof CompositeName) {
                        logger.log(BasicLevel.DEBUG, "compositeName : "
                                + ((CompositeName) o).getFQName());
                    } else
                        logger.log(BasicLevel.DEBUG, "metaObject : " + o);
                }

            //Verify the meta information
            //jormcompiler.verifyMetaInfo(c);

            //Launch the jorm generation
            c = jormcompiler.generateFiles(c);

            if (logger.isLoggable(BasicLevel.DEBUG))
                for (Iterator it = c.iterator(); it.hasNext();)
                    logger.log(BasicLevel.DEBUG, "Generated files: " + it.next());

            //Serialize the meta information for each xml descriptor
            for (Iterator itDesc = scp.xmldescriptor.values().iterator(); itDesc.hasNext();) {
                serializeJormMI((SpeedoXMLDescriptor) itDesc.next());
            }
        } catch (PException e) {
            throw new SpeedoException("Error during Jorm generation", e);
        }
    }

    /**
     * Verifies that all persistent capable field for jdo is described in JORM.
     *
     * @exception SpeedoXMLError if JORM an JDO MetaData are not coherent
     */
    protected void isCompatible() throws SpeedoException {

        List except = new ArrayList();

        Manager manager = jormcompiler.getMIManager();
        for (Iterator itDesc = scp.xmldescriptor.values().iterator(); itDesc.hasNext();) {
            SpeedoXMLDescriptor desc = (SpeedoXMLDescriptor) itDesc.next();
            List scs = desc.getSpeedoClasses();
            for (int i=0; i<scs.size(); i++) {
                compareClass((SpeedoClass) scs.get(i), manager, except);
            }
        }
        if (!except.isEmpty() && logger.isLoggable(BasicLevel.ERROR))
            for (Iterator it = except.iterator(); it.hasNext();)
                logger.log(BasicLevel.ERROR, it.next());
    }

    protected void compareClass(SpeedoClass clas,
                                Manager manager,
                                List except) throws SpeedoException {
        boolean debug = logger.isLoggable(BasicLevel.DEBUG);
        if (debug) {
            logger.log(BasicLevel.DEBUG,
                    "Verify the definition of the class " + clas.getFQName());
        }
        Class classJorm = manager.getClass(clas.getFQName());
        if (classJorm == null) {
            throw new SpeedoException("Jorm description of the class '"
                    + clas.getFQName() + "' is not availlable");
        }
        NameDef pName = getClassNameDef(classJorm);

        if (classJorm == null) {
            if (!scp.failsonerror) {
                except.add("Class '" + clas.name + "' not defined in JORM metadata");
                return;
            } else
                throw new SpeedoXMLError("Class '" + clas.name + "' not defined in JORM metadata");
        }

        // Comparison from JDO
        clas.jormclass = classJorm;
        for (Iterator efield = clas.jdoField.values().iterator(); efield.hasNext();) {
            SpeedoField field = (SpeedoField) efield.next();
            String fieldName = field.name;

            // Fields with this persistent modifier shouldn't stay in JDO Metadata
            if (field.persistenceModifier == SpeedoModifier.none)
                continue;

            TypedElement tElem = classJorm.getTypedElement(fieldName);
            if (tElem == null)
                if (!scp.failsonerror) {
                    except.add("Field '" + fieldName + "' not defined in JORM metadata of the class '" + clas.getFQName() + "'");
                    continue;
                } else
                    throw new SpeedoXMLError("Field '" + fieldName + "' not defined in JORM metadata of the class '" + clas.getFQName() + "'");

            // for application identity key fields have to be compared
            if (field.primaryKey) {
                boolean found = false;
                for (Iterator it = pName.iterateField(); it.hasNext() && !found;) {
                    found = fieldName.equals(it.next());
                }
                if (!found) {
                    except.add("Field '" + fieldName + "' not defined in Class NameDef's Fields of the class '" + clas.getFQName() + "'");
                }
            }

            if (tElem instanceof GenClassRef && field.jdoTuple == null)
                if (!scp.failsonerror)
                    except.add("Field '" + fieldName + "' should be a tuple in JDO metadata of the class '" + clas.getFQName() + "'");
                else
                    throw new SpeedoXMLError("field '" + fieldName + "' should be a tuple in JDO metadata of the class '" + clas.getFQName() + "'");
        }

        // Comparison from JORM
        for (Iterator jormfield = classJorm.iterateField(); jormfield.hasNext();) {
            TypedElement te = (TypedElement) jormfield.next();
            String fieldName = te.getName();
            if (!clas.jdoField.containsKey(fieldName)
                    && !isContainerIdField(classJorm, te, clas))
                if (!scp.failsonerror)
                    except.add("Field '" + fieldName + "' of the class '"
                            + clas.getFQName() + "' is not defined in the '"
                            + clas.jdoPackage.jdoXMLDescriptor.xmlFile
                            + "' file (found: " + clas.jdoField.keySet() + ").");
                else
                    throw new SpeedoXMLError("field " + fieldName + " not defined in JDO metadata");
        }

        // for application identity key fields have to be compared
        if (clas.identityType == SpeedoIdentity.USER_ID) {
            boolean found = false;
            for (Iterator it = pName.iterateField(); it.hasNext() && !found;) {
                String fn = (String) it.next();
                SpeedoField sf = (SpeedoField) clas.jdoField.get(fn);
                if (sf == null) {
                    except.add("Field " + fn + " defined in the "
                            + clas.name
                            + ".pd file but not availlable in the persistent class.");
                } else if (!sf.primaryKey) {
                    except.add("Field " + fn + " defined in the "
                            + clas.name
                            + ".pd file but is not marked as a primary key field in the "
                            + clas.jdoPackage.jdoXMLDescriptor.xmlFile
                            + " file.");
                }
            }
        }
    }

    private boolean isContainerIdField(Class clazz,
                                       TypedElement te,
                                       SpeedoClass sc) throws SpeedoException {
        return sc.identityType == SpeedoIdentity.CONTAINER_ID
                && getClassNameDef(clazz).getNameRef()
                .getProjection().containsValue(te.getName());
    }


    private Mapping getMapping(Class clazz) throws SpeedoException {
        ClassProject cp = clazz.getClassProject(scp.projectName);
        if (cp == null) {
            throw new SpeedoException("No classproject found for the class "
                    + clazz.getFQName() + " and  the project " + scp.projectName);
        }
        int idx =  scp.mapperName.indexOf('.');
        Mapping m = cp.getMapping(idx == -1
                ? scp.mapperName
                : scp.mapperName.substring(0, idx));
        if (m == null) {
            throw new SpeedoException("No mapping found for the class "
                    + clazz.getFQName() + ",  the project " + scp.projectName
                    + " and  the mapper " + scp.mapperName);
        }
        return m;
    }

    private NameDef getClassNameDef(Class clazz) throws SpeedoException {
        return (NameDef) getMapping(clazz).getClassMapping()
                .getIdentifierMapping().getLinkedMO();
    }

    private void serializeJormMI(SpeedoXMLDescriptor desc)
            throws SpeedoException {
        // Calculate the list of the Jorm Meta Object which must be serialized
        Set mos = new HashSet(desc.jdoPackage.size());
        for (Iterator itPack = desc.jdoPackage.values().iterator(); itPack.hasNext();) {
            SpeedoPackage sp = (SpeedoPackage) itPack.next();
            for (Iterator itclass = sp.jdoClass.values().iterator(); itclass.hasNext();) {
                SpeedoClass jdoClass = (SpeedoClass) itclass.next();
                if (jdoClass.jormclass == null)
                    throw new SpeedoException(
                            "The Jorm meta information of the jdo descriptor "
                            + desc.xmlFile + " is not complete: class "
                            + jdoClass.name + " has not been found");
                // The class meta object
                addMOClass(jdoClass.jormclass, mos);

                // Add the CompositeName which could be used in the class name def
                for (Iterator it = jdoClass.jormclass.iterateNameDef(); it.hasNext();) {
                    NameDef nd = (NameDef) it.next();
                    if (nd.isNameRef()) {
                        mos.add(nd.getNameRef().getCompositeName());
                    }
                }

                //The compositeName used for the reference will be added by the
                // name definition of the referenced class.
            }
        }

        // Serialize the list in the file
        String miFileName = scp.output + File.separator + desc.xmlFile.substring(0, desc.xmlFile.length() - 4)
                + ".jmi";
        try {
            ObjectOutputStream out =
                    new ObjectOutputStream(new FileOutputStream(miFileName));
            out.writeObject(mos);
            out.close();
        } catch (IOException io) {
            throw new SpeedoException("IO Exception, impossible to write the " +
                    "meta information in the file " + miFileName, io);
        }

    }

    private void addMOClass(Class mo, Set mos) {
        mos.add(mo);
        for(Iterator it = mo.iterateInheritedClass();it.hasNext();) {
            addMOClass((Class) it.next(), mos);
        }
    }
}
