/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: P. Dechamboux, Y. Devaux, F. Dhumeaux.
 *
 */

package org.objectweb.speedo.metadata;

import org.objectweb.asm.Constants;
import org.objectweb.asm.Type;
import org.objectweb.speedo.generation.enhancer.Util;
import org.objectweb.speedo.api.SpeedoProperties;

/**
 * This class corresponds to the description of persistent capable fields.
 * @author	Y. Devaux
 */
public class SpeedoField extends SpeedoElement {
    /**
     * SpeedoAccessor name.
     */
    public String name;

    public int access;

    public String desc;

    public int number;

    /**
     * Attribute persistence-modifier in the XML file.
     */
    public byte persistenceModifier = SpeedoModifier.missing;

    /**
     * Boolean indicating if the field is a primary key.
     */
    public boolean primaryKey = false;

    /**
     * Attribute null-value in the XML file.
     */
    public byte nullValue = SpeedoNullValue.none;

    /**
     * This boolean indicates if the field is in the default fetch group.
     */
    public boolean defaultFetchGroup = true;

    /**
     * Attribute embedded of the XML file.
     */
    public boolean embedded = true;

    /**
     * Type of the tuple represented by this SpeedoField if it is a tuple
     */
    public SpeedoTuple jdoTuple;

    /**
     * Description of the class containing the field.
     */
    public SpeedoClass jdoClass;

    /**
     * True if an getter was put in the code of the SpeedoClass
     */
    public boolean hasGetter = false;

    /**
     * True if an setter was put in the code of the SpeedoClass
     */
    public boolean hasSetter = false;

    /**
     * Transforms a SpeedoField into a String.
     * @return the String corresponding to the SpeedoField.
     */
    public String toString() {
        String s = "\n field name : " + name + ", persistenceModifier : "
                + persistenceModifier + ", primaryKey : " + primaryKey
                + ", nullValue : " + nullValue + ", defaultFetchGroup : "
                + defaultFetchGroup + ", embedded : " + embedded;
        if (jdoTuple != null)
            s = s + jdoTuple.toString();
        return s;
    }

    /**
     * Returns the field's signature with a public modifier.
     * @return the String containing the field' signature with a public modifier.
     */
    public String publicSignature() {
        int publicAccess = access;
        publicAccess &= ~(Constants.ACC_PRIVATE | Constants.ACC_PROTECTED);
        publicAccess |= Constants.ACC_PUBLIC;
        return Util.modifier(publicAccess) + type() + " " + name;
    }

    /**
     * Returns the field's signature with a private modifier.
     * @return the String containing the field' signature with a private modifier.
     */
    public String privateSignature() {
        int privateAccess = access;
        privateAccess &= ~(Constants.ACC_PUBLIC | Constants.ACC_PROTECTED);
        privateAccess |= Constants.ACC_PRIVATE;
        return Util.modifier(privateAccess) + type() + " " + name;
    }

    /**
     * Returns the field's modifier
     * @return the String representing the modifier (<code>public</code>,
     * <code>private</code>, <code>protected</code>)
     */
    public String modifier() {
        return Util.modifier(access);
    }

    /**
     * Returns the type of the field.
     * @return type of the field.
     */
    public String type() {
        return Util.type(Type.getType(desc));
    }
}
