/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: P. Dechamboux, Y. Devaux, F. Dhumeaux.
 *
 */

package org.objectweb.speedo.metadata;

import org.objectweb.speedo.api.SpeedoXMLError;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.util.monolog.api.Logger;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Describes a package which contains persistence capable classes.
 * @author	Y. Devaux
 */
public class SpeedoPackage extends SpeedoElement {
    /**
     * Package name.
     */
    public String name;

    /**
     * Persistence capable classes descriptors of the package. The HashMap key
     * is the class name.
     */
    public Map jdoClass = new HashMap();

    /**
     * Descriptor to which this package is associated.
     */
    public SpeedoXMLDescriptor jdoXMLDescriptor;

    /**
     * Transforms a SpeedoPackage into a String.
     * @return the corresponding String.
     */
    public String toString() {
        String s = "package  name : " + name;
        Iterator it = jdoClass.values().iterator();
        while (it.hasNext()) {
            s = s + "  " + it.next().toString();
        }
        return s;
    }

    /**
     * Adds a class descriptor to the package descriptor.
     * @param classe class to add.
     * @param failsOnError if an error provoques an exception or a warning message.
     * @param logger logger where to put warning message.
     * @exception SpeedoException If a field of the class descriptor is already
     * defined into the package descriptor.
     */
    public void add(Object classe, boolean failsOnError, Logger logger) throws SpeedoException {
        SpeedoClass c = (SpeedoClass) classe;
        //the class already exists
        if (jdoClass.containsKey(c.name)) {
            //we add its fields
            SpeedoClass cref = (SpeedoClass) jdoClass.get(c.name);
            Iterator it = c.jdoField.values().iterator();
            while (it.hasNext()) {
                cref.add(it.next(), failsOnError, logger);
            }
        }
        //we add the entire class;
        else {
            c.jdoPackage = this;
            jdoClass.put(c.name, classe);
        }
    }
} // end SpeedoPackage
