/**
 * copyright area
 */
package org.objectweb.speedo.runtime;

import org.objectweb.fractal.api.control.UserBindingController;
import org.objectweb.fractal.api.ComponentIdentity;
import org.objectweb.fractal.api.InterfaceReference;
import org.objectweb.fractal.julia.control.content.SubComponentController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.perseus.cache.api.CacheException;
import org.objectweb.perseus.cache.api.CacheManager;
import org.objectweb.perseus.cache.replacement.api.ReplacementManager;
import org.objectweb.perseus.pool.api.Pool;
import org.objectweb.perseus.pool.api.PoolMatchFactory;
import org.objectweb.perseus.pool.api.PoolResource;
import org.objectweb.perseus.persistence.api.TransactionalPersistenceManager;
import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.speedo.api.LifeCycle;
import org.objectweb.speedo.api.SpeedoAccessor;
import org.objectweb.speedo.api.SpeedoProxy;
import org.objectweb.speedo.api.ProxyManager;
import org.objectweb.speedo.api.ProxyManagerSwitch;
import org.objectweb.speedo.api.Transaction;
import org.objectweb.speedo.api.JormFactory;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.api.SpeedoGenClassProxy;
import org.objectweb.speedo.runtime.query.SpeedoExtent;
import org.objectweb.speedo.runtime.query.SpeedoQuery;
import org.objectweb.speedo.runtime.query.api.QueryDefinition;
import org.objectweb.speedo.runtime.query.api.QueryManager;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import javax.jdo.Extent;
import javax.jdo.InstanceCallbacks;
import javax.jdo.JDODataStoreException;
import javax.jdo.JDOException;
import javax.jdo.JDOFatalUserException;
import javax.jdo.JDOUnsupportedOptionException;
import javax.jdo.JDOUserException;
import javax.jdo.PersistenceManager;
import javax.jdo.PersistenceManagerFactory;
import javax.jdo.Query;
import javax.jdo.JDOFatalInternalException;
import java.util.Collection;
import java.util.Iterator;
import java.util.ArrayList;

/**
 * @author S.Chassande-Barrioz
 */
public class SpeedoPersistenceManager
        implements PersistenceManager,
        ProxyManager,
        PoolResource,
        PoolMatchFactory,
        UserBindingController {

    public final static String PERSISTENCE_MANAGER_FACTORY_BINDING = "persistence-manager-factory";
    public final static String TRANSACTIONAL_PERSISTENCE_MANAGER_BINDING = "transactional-persistence-manager";
    public final static String QUERY_MANAGER_BINDING = "query-manager";
    public final static String CACHE_BINDING = "cache-manager";
    public final static String REPLACEMENT_BINDING = "replacement-manager";
    public final static String CONTAINER_NAMING_MANGER_BINDING = "container-naming-manager";
    public final static String USER_NAMING_MANGER_BINDING = "user-naming-manager";
    public final static String TRANSACTION_BINDING = "transaction";
    public final static String PROXY_MANAGER_SWITCH_BINDING = "proxy-manager-switch";
    public final static String MAPPER_BINDING = "mapper";
    public final static String QUERIES_POOL_BINDING = "query-pool";
    public final static String PM_POOL_BINDING = "persistence-manager-pool";
    public final static String JORM_FACTORY_BINDING = "jorm-factory";
    public final static String COMPONENT_IDENTITY_BINDING = "component-identity";
    public final static String LOGGER_FACTORY_BINDING = "logger-factory";
    public final static String LOGGER_NAME
            = "org.objectweb.speedo.rt.SpeedoPersistenceManager";

    private LoggerFactory loggerFactory = null;

    private PersistenceManagerFactory pmf = null;
    private PersistenceManagerFactory _pmf = null;

    /**
     * is the information permiting the access to the data store
     */
    private Object connectionSpec = null;

    /**
     * This fields is the cache used in the PersistentManager
     */
    private CacheManager cache = null;

    private ReplacementManager replacementManager = null;

    /**
     * The manager of SpeedoQuery. It contains the optimized queries already
     * used.
     */
    private QueryManager queryManager = null;

    /**
     * The pool of SpeedoPersitenceManager. This field will be used to release
     * this instance in the pool.
     */
    private Pool managedPM = null;

    /**
     * The jorm mapper. It represents a data source.
     */
    private PMapper mapper = null;

    /**
     * This is the pool of query.
     */
    private Pool queries = null;

    /**
     * It maintains the association between a context (thread) and a
     * ProxyManager
     */
    private ProxyManagerSwitch pms = null;

    /**
     * The transaction associated to this proxy manager.
     */
    private Transaction tx;

    private TransactionalPersistenceManager tpm = null;

    private JormFactory jf = null;

    /**
     * Indicates if the proxy manager is closed.
     */
    private boolean isClosed = false;

    private Logger logger = null;

    /**
     * A user object
     */
    Object userObject = null;

    /**
     * indicates is the ProxyManager can be used by several thread
     */
    boolean multithreaded = true;

    boolean ignoreCache = true;

    Object thisPR = null;
    Object thisPM = null;

    public SpeedoPersistenceManager() {
    }

    // IMPLEMENTATION OF THE UserBindingController INTERFACE //
    //-------------------------------------------------------//

    public Object getFcBindings(String s) {
        if (LOGGER_FACTORY_BINDING.equals(s)) {
            return loggerFactory;
        } else if (PERSISTENCE_MANAGER_FACTORY_BINDING.equals(s)) {
            return pmf;
        } else if (QUERY_MANAGER_BINDING.equals(s)) {
            return queryManager;
        } else if (TRANSACTIONAL_PERSISTENCE_MANAGER_BINDING.equals(s)) {
            return tpm;
        } else if (TRANSACTION_BINDING.equals(s)) {
            return tx;
        } else if (PROXY_MANAGER_SWITCH_BINDING.equals(s)) {
            return pms;
        } else if (PM_POOL_BINDING.equals(s)) {
            return managedPM;
        } else if (QUERIES_POOL_BINDING.equals(s)) {
            return queries;
        } else if (CACHE_BINDING.equals(s)) {
            return cache;
        } else if (REPLACEMENT_BINDING.equals(s)) {
            return replacementManager;
        } else if (MAPPER_BINDING.equals(s)) {
            return mapper;
        } else if (JORM_FACTORY_BINDING.equals(s)) {
            return jf;
        } else {
            return null;
        }
    }

    public void addFcBinding(String s, Object o) {
        if (LOGGER_FACTORY_BINDING.equals(s)) {
            loggerFactory = (LoggerFactory) o;
            if (loggerFactory != null)
                logger = loggerFactory.getLogger(LOGGER_NAME);
        } else if (PERSISTENCE_MANAGER_FACTORY_BINDING.equals(s)) {
            pmf = (PersistenceManagerFactory) o;
        } else if (QUERY_MANAGER_BINDING.equals(s)) {
            queryManager = (QueryManager) o;
        } else if (TRANSACTIONAL_PERSISTENCE_MANAGER_BINDING.equals(s)) {
            tpm = (TransactionalPersistenceManager) o;
        } else if (TRANSACTION_BINDING.equals(s)) {
            tx = (Transaction) o;
        } else if (PROXY_MANAGER_SWITCH_BINDING.equals(s)) {
            pms = (ProxyManagerSwitch) o;
        } else if (PM_POOL_BINDING.equals(s)) {
            managedPM = (Pool) o;
        } else if (QUERIES_POOL_BINDING.equals(s)) {
            queries = (Pool) o;
        } else if (CACHE_BINDING.equals(s)) {
            cache = (CacheManager) o;
        } else if (REPLACEMENT_BINDING.equals(s)) {
            replacementManager = (ReplacementManager) o;
        } else if (MAPPER_BINDING.equals(s)) {
            mapper = (PMapper) o;
        } else if (JORM_FACTORY_BINDING.equals(s)) {
            jf = (JormFactory) o;
        } else if (COMPONENT_IDENTITY_BINDING.equals(s)) {
            thisPR = o;
            thisPM = o;
        }
    }

    public void removeFcBinding(String s, Object serverItf) {
        if (LOGGER_FACTORY_BINDING.equals(s)) {
            loggerFactory = null;
        } else if (PERSISTENCE_MANAGER_FACTORY_BINDING.equals(s))
            pmf = null;
        else if (TRANSACTIONAL_PERSISTENCE_MANAGER_BINDING.equals(s))
            tpm = null;
        else if (TRANSACTION_BINDING.equals(s))
            tx = null;
        else if (PROXY_MANAGER_SWITCH_BINDING.equals(s))
            pms = null;
        else if (QUERY_MANAGER_BINDING.equals(s))
            queryManager = null;
        else if (PM_POOL_BINDING.equals(s))
            managedPM = null;
        else if (QUERIES_POOL_BINDING.equals(s))
            queries = null;
        else if (CACHE_BINDING.equals(s))
            cache = null;
        else if (REPLACEMENT_BINDING.equals(s))
            replacementManager = null;
        else if (MAPPER_BINDING.equals(s))
            mapper = null;
        else if (JORM_FACTORY_BINDING.equals(s))
            jf = null;
    }

    // IMPLEMENTATION OF THE PoolResource INTERFACE //
    //----------------------------------------------//

    /**
     * <b>destroy</b> is called when this resource is removed from a Pool.
     */
    public void destroy() throws Exception {
    }


    // IMPLEMENTATION OF THE PoolMatchFactory INTERFACE //
    //--------------------------------------------------//

    public PoolResource createResource(Object hints) throws Exception {
        SpeedoQuery sq = new SpeedoQuery();
        sq.setProxyManager(this);
        sq.setQueryManager(queryManager);
        sq.setLoggerFactory(loggerFactory);
        sq.setPool(queries);
        if (hints != null)
            sq.defineWith((QueryDefinition) hints);
        return sq;
    }

    public boolean matchResource(PoolResource pr, Object hints) {
        ((SpeedoQuery) pr).defineWith((QueryDefinition) hints);
        return true;
    }


    // IMPLEMENTATION OF THE PersistentManager INTERFACE //
    //---------------------------------------------------//

    public java.lang.Class getObjectIdClass(Class cls) {
        try {
            return jf.getPBinder(cls.getName()).getNull().getClass();
        } catch (PException e) {
            throw new JDOException("", ExceptionHelper.getNested(e));
        }
    }

    /**
     * A PersistenceManager instance can be used until it is closed.
     * @return true if this PersistenceManager has been closed
     * @see #close()
     */
    public boolean isClosed() {
        return isClosed;
    }

    /**
     * This method closes the PersistenceManager.
     * @exception javax.jdo.JDOUserException if the transaction associated to
     * the persistence manager is active.
     * @exception javax.jdo.JDOFatalDataStoreException if there is a problem while
     * releasing the persistence manager.
     */
    public synchronized void close() {
        if (isClosed)
            return;
        try {
            tpm.close(tx);
        } catch (PersistenceException e) {
            throw new JDOFatalInternalException(
                    "Impossible to close the persistence manager",
                    ExceptionHelper.getNested(e));
        }
        isClosed = true;
        userObject = null;
        if (!(thisPR instanceof PoolResource)) {
            SubComponentController parentCtrl = (SubComponentController)
                    ((ComponentIdentity) thisPR).getFcInterface("subcomponent-controller");
            thisPR = parentCtrl.getFcParents()[0].getFcInterface("pool-resource");
        }
        if (!pms.remove(getThisPM()))
            throw new JDOFatalInternalException(
                    "The PM was not associated to the context");

        //Forget the information to access to the data store.
        connectionSpec = null;

        //Release the PM in the pool
        try {
            managedPM.releaseResource((PoolResource) thisPR);
        } catch (Exception e) {
            throw new JDOFatalInternalException(
                    "Impossible to release this PersistenceManager in the pool.",
                    new Exception[]{ExceptionHelper.getNested(e)});
        } finally {
            if (logger.isLoggable(BasicLevel.INFO))
                logger.log(BasicLevel.INFO, "Persistence Manager closed");
        }
    }

    /**
     * Return the Transaction instance associated with a PersistenceManager.
     * @return the Transaction associated with this
     * PersistenceManager.
     */
    public javax.jdo.Transaction currentTransaction() {
        assertIsOpen();
        return tx;
    }

    /**
     * Mark an instance as no longer needed in the cache.
     * @param o the instance to evict from the cache.
     */
    public void evict(Object o) {
        SpeedoProxy sp = (SpeedoProxy) o;
        if (tx.isActive() && sp.jdoIsDirty())
            throw new JDOUserException("Impossible to evict a dirty " +
                    "instance attached to an active transaction");
        tx.unbind(sp.getPName());
        try {
            replacementManager.unbind(sp);
        } catch (CacheException e) {
            throw new JDOUserException(
                "Impossible to evict the persistent object from the cache", e);
        }
    }

    /** Mark an array of instances as no longer needed in the cache.
     * @see #evict(Object pc)
     * @param pcs the array of instances to evict from the cache.
     * @exception JDOUserException if some instances cannot be removed.
     */
    public void evictAll(Object[] pcs) {
        if (pcs == null)
            return;
        Throwable[] th = new Throwable[pcs.length];
        int lg = 0;
        for (int i = 0; i < pcs.length; i++) {
            try {
                evict(pcs[i]);
            } catch (Throwable e) {
                th[lg] = e;
                lg++;
            }
        }
        if (lg > 0) {
            Throwable[] tfin = new Throwable[lg];
            System.arraycopy(th, 0, tfin, 0, lg);
            throw new JDOUserException("Impossible to evict", tfin);
        }
    }

    /** Mark a Set of instances as no longer needed in the cache.
     * @see #evict(Object pc)
     * @param pcs the Set of instance to evict from the cache.
     */
    public void evictAll(Collection pcs) {
        if (pcs == null)
            return;
        evictAll(pcs.toArray());
    }

    /** Mark all persistent-nontransactional instances as no longer needed
     * in the cache. It transitions all persistent-nontransactional instances to
     * hollow. Transactional instances are subject to eviction based on the
     * RetainValues setting.
     * @see #evict(Object pc)
     */
    public void evictAll() {
        evict(tx.entries());
    }

    /** Refresh the state of the instance from the data store.
     * <P>In an optimistic transaction, the state of instances in the cache
     * might not match the state in the data store.  This method is used to
     * reload the state of the instance from the data store so that a subsequent
     * commit is more likely to succeed.
     * <P>Outside a transaction, this method will refresh nontransactional
     * state.
     * @param o the instance to refresh.
     */
    public void refresh(Object o) {
        SpeedoProxy pc = (SpeedoProxy) o;
        assertIsOpen();
        assertIsSpeedoProxy(pc);
        assertPersistenceManager(pc);
        if (!pc.jdoIsPersistent())
            throw new JDOUserException("Refresh on a transient instance.");
        PName pn = pc.getPName();

        if (cache.lookup(pn) == null) {
            try {
                cache.bind(pn, pc);
            } catch (CacheException e) {
                throw new JDOException(
                        "Impossible to add an element in the cache", e);
            }
        }

        byte s = pc.getStatus();
        if (s == PBinding.LIFECYCLE_NOTBOUND
                || s == PBinding.LIFECYCLE_NEWTOWRITE
                || s == PBinding.LIFECYCLE_DELTOWRITE) {
            //instance deleted or persistent new
            return;
        }

        readObject(pc);

        //chgt status
        if (currentTransaction().getOptimistic())
            pc.jdoChangeStatus(LifeCycle.ACTION_REFRESH_ACTIVEOPTIMISTICTRANSACTION);
        else
            pc.jdoChangeStatus(LifeCycle.ACTION_REFRESH_ACTIVEDATASTORETRANSACTION);
    }

    /** Refresh the state of an array of instances from the data store.
     * @see #refresh(Object pc)
     * @param pcs the array of instances to refresh.
     * object.
     */
    public void refreshAll(Object[] pcs) {
        if (pcs == null || pcs.length == 0)
            return;
        Throwable[] th = new Throwable[pcs.length];
        int lg = 0;
        for (int i = 0; i < pcs.length; i++) {
            try {
                refresh(pcs[i]);
            } catch (Throwable e) {
                th[lg] = e;
                lg++;
            }
        }
        if (lg > 0) {
            Throwable[] tfin = new Throwable[lg];
            System.arraycopy(th, 0, tfin, 0, lg);
            throw new JDOUserException("Impossible to refresh", tfin);
        }
    }

    /** Refresh the state of a Set of instances from the data store.
     * @see #refresh(Object pc)
     * @param pcs the Set of instances to refresh.
     */
    public void refreshAll(Collection pcs) {
        if (pcs == null || pcs.size() == 0)
            return;
        refreshAll(pcs.toArray());
    }

    /** Refresh the state of all applicable instances from the data store.
     * <P>If called with an active transaction, all transactional instances
     * will be refreshed.  If called outside an active transaction, all
     * nontransactional instances will be refreshed.
     * @see #refresh(Object pc)
     */
    public void refreshAll() {
        refreshAll(tx.entries());
    }

    /**
     * Create a new Query with no elements.
     * @return the new Query.
     */
    public Query newQuery() {
        assertIsOpen();
        try {
            return (Query) queries.getResource(null);
        } catch (Exception e) {
            throw new JDOException("Impossible to fetch a query from the pool",
                    ExceptionHelper.getNested(e));
        }
    }

    /**
     * Create a new Query using elements from another Query.
     * The other Query must have been created by the same JDO implementation.
     * It might be active in a different PersistenceManager or might have been
     * serialized and restored.
     * <P>All of the settings of the other Query are copied to this Query,
     * except for the candidate Set or Extent.
     * @return the new Query
     * @param compiled another Query from the same JDO implementation
     */
    public Query newQuery(Object compiled) {
        assertIsOpen();
        try {
            return (Query) queries.getResource(compiled);
        } catch (Exception e) {
            throw new JDOException("Impossible to fetch a query from the pool",
                    ExceptionHelper.getNested(e));
        }
    }

    /**
     * Create a new Query using the specified language.
     * @param language the language of the query parameter
     * @param query the query, which is of a form determined by the language
     * @return the new Query
     */
    public Query newQuery(String language, Object query) {
        assertIsOpen();
        if (language.compareTo("java.jdo.query.toVerify") == 0) {
            try {
                return (Query) queries.getResource(query);
            } catch (Exception e) {
                throw new JDOException("Impossible to fetch a query from the pool",
                        ExceptionHelper.getNested(e));
            }
        } else
            throw new JDOUnsupportedOptionException("Language \"" + language
                    + "\"" + " is not supported by this jdo implementation");
    }

    /**
     * Create a new Query specifying the Class of the candidate instances.
     * @param cls the Class of the candidate instances
     * @return the new Query
     */
    public Query newQuery(Class cls) {
        assertIsOpen();
        Query q = null;

        try {
            q = (Query) queries.getResource(null);
        } catch (Exception e) {
            throw new JDOException("Impossible to fetch a query from the pool",
                    ExceptionHelper.getNested(e));
        }
        q.setClass(cls);
        if (getObjectIdClass(cls) == null)
            throw new JDOUnsupportedOptionException(
                    "There is a problem with the specified class");
        return q;
    }

    public Query newQuery(Extent extent) {
//TODO: support the creation of query with an extent
        return null;
    }

    public Query newQuery(Extent extent, String s) {
//TODO: support the creation of query with an extent
        return null;
    }

    /**
     * Create a new Query with the Class of the candidate instances and
     * candidate Set.
     * @param cls the Class of results
     * @param cln the Set of candidate instances
     * @return the new Query
     */
    public Query newQuery(Class cls, Collection cln) {
        assertIsOpen();
        Query q = null;
        try {
            q = (Query) queries.getResource(null);
        } catch (Exception e) {
            throw new JDOException("Impossible to fetch a query from the pool",
                    ExceptionHelper.getNested(e));
        }
        q.setClass(cls);
        q.setCandidates(cln);
        return q;
    }

    /**
     * Create a new Query with the Class of the candidate instances and filter.
     * @param cls the Class of results
     * @param filter the filter for candidate instances
     * @return the new Query
     */
    public Query newQuery(Class cls, String filter) {
        assertIsOpen();
        Query q = null;
        try {
            q = (Query) queries.getResource(null);
        } catch (Exception e) {
            throw new JDOException("Impossible to fetch a query from the pool",
                    ExceptionHelper.getNested(e));
        }
        q.setClass(cls);
        q.setFilter(filter);
        return q;
    }

    /**
     * Create a new Query with the Class of the candidate instances,
     * candidate Set, and filter.
     * @param cls the Class of candidate instances
     * @param cln the Set of candidate instances
     * @param filter the filter for candidate instances
     * @return the new Query
     */
    public Query newQuery(Class cls, Collection cln, String filter) {
        assertIsOpen();
        Query q = null;
        try {
            q = (Query) queries.getResource(null);
        } catch (Exception e) {
            throw new JDOException("Impossible to fetch a query from the pool",
                    ExceptionHelper.getNested(e));
        }
        q.setClass(cls);
        q.setCandidates(cln);
        q.setFilter(filter);
        return q;
    }

    /** Not implemented. The PersistenceManager manages a collection of
     * instances in the data store based on the class of the instances. This
     * method returns a Extent of instances in the data store that might be
     * iterated or given to a Query. The Extent itself might not reference any
     * instances, but only hold the class name and an
     * indicator whether subclasses are included in the Extent.
     * @param persistenceCapableClass Class of instances
     * @param subclasses whether to include instances of subclasses
     * @return an Extent of the specified Class
     */
    public Extent getExtent(Class persistenceCapableClass,
                            boolean subclasses) {
        assertIsOpen();
        return new SpeedoExtent(persistenceCapableClass, subclasses, this);
    }

    /** This method locates a persistent instance in the cache of instances
     * managed by this PersistenceManager.  The getObjectById method attempts
     * to find an instance in the cache with the specified JDO identity.
     * The oid parameter object might have been returned by an earlier call
     * to getObjectId or getTransactionalObjectId, or might have been
     * constructed by the application.
     * @see #getObjectId(Object pc)
     * @see #getTransactionalObjectId(Object pc)
     * @return the PersistenceCapable instance with the specified
     * ObjectId
     * @param oid an ObjectId
     * @param validate if the existence of the instance is to be validated. The
     * flag is ignored in this implementation.
     */
    public Object getObjectById(Object oid, boolean validate) {
        assertIsOpen();
        if (oid == null)
            return null;
        SpeedoAccessor sa = null;
        try {
            sa = (SpeedoAccessor) tpm.readIntention(tx, jf.decode(oid));
        } catch (Exception e) {
            throw new JDOException("", ExceptionHelper.getNested(e));
        }
        SpeedoProxy sp = sa.jdoGetSpeedoProxy();
        if (!sp.jdoIsActive()) {
            activeProxy(sp);
            sp.jdoSetStatus(LifeCycle.PERSISTENT_CLEAN);
        }
        return sp;
    }

    /** The ObjectId returned by this method represents the JDO identity of
     * the instance.  The ObjectId is a copy (clone) of the internal state
     * of the instance, and changing it does not affect the JDO identity of
     * the instance.
     * @see #getTransactionalObjectId(Object pc)
     * @see #getObjectById(Object oid, boolean validate)
     * @param pc the PersistenceCapable instance
     * @return the ObjectId of the instance
     */
    public Object getObjectId(Object pc) {
        assertIsOpen();
        //the object must be persistence capable. otherwise, it doesn't have an
        //objectid
        try {
            assertIsSpeedoProxy(pc);
        } catch (Exception e) {
            return null;
        }
        SpeedoProxy sp = (SpeedoProxy) pc;
        if (!sp.jdoIsPersistent())
            throw new JDOUserException("Object non persistent.");
        assertPersistenceManager(sp);
        try {
            return jf.encode(sp.getPName());
        } catch (PException e) {
            throw new JDOException(
                    "Problem while encoding persistent name.",
                    new Exception[]{ExceptionHelper.getNested(e)});
        }
    }

    /** Not implemented. The ObjectId returned by this method represents the JDO
     * identity of the instance.  The ObjectId is a copy (clone) of the internal
     * state of the instance, and changing it does not affect the JDO identity
     * of the instance.
     * <P>If the object identity is being changed in the transaction, by the
     * application modifying one or more of the application key fields,
     * then this method returns the current identity in the transaction.
     * <P>If there is no transaction in progress, or if none of the key fields
     * is being modified, then this method will return the same value as
     * getObjectId.
     * @see #getObjectId(Object pc)
     * @see #getObjectById(Object oid, boolean validate)
     * @param o a PersistenceCapable instance
     * @return the ObjectId of the instance
     */
    public Object getTransactionalObjectId(Object o) {
        assertIsOpen();
        throw new JDOUnsupportedOptionException("Not implemented.");
    }

    /** Make the transient instance persistent in this PersistenceManager.
     * This method must be called in an active transaction.
     * The PersistenceManager assigns an ObjectId to the instance and
     * transitions it to persistent-new.
     * The instance will be managed in the Extent associated with its Class.
     * The instance will be put into the data store at commit.
     * The closure of instances of PersistenceCapable classes
     * reachable from persistent
     * fields will be made persistent at commit.  [This is known as
     * persistence by reachability.]
     * @param o a transient instance of a Class that implements
     * PersistenceCapable
     */
    public void makePersistent(Object o) {
        assertIsOpen();
        assertIsSpeedoProxy(o);
        SpeedoProxy sp = (SpeedoProxy) o;
        if (sp.jdoIsPersistent())
            return;
        logger.log(BasicLevel.DEBUG, "Make persistent the proxy: " + sp);
        try {
            activeProxy(sp);
            Object hints = sp.jdoGePNameHints();
            if (hints == null) {
                tpm.export(tx, sp);
            } else {
                tpm.export(tx, sp, hints);
            }
            //Now a PName has been assigned, call the speedoProxy to make
            // persistent the field of the instance.
            sp.jdoMakePersistent(this);
            sp.jdoChangeStatus(LifeCycle.ACTION_MAKEPERSISTENT);
        } catch (Exception e) {
            throw new JDODataStoreException(
                    "Problem while making persistent.",
                    new Exception[]{ExceptionHelper.getNested(e)});
        }
        logger.log(BasicLevel.DEBUG, "The instance is now persistent: " + sp);
    }

    /** Make an array of instances persistent.
     * @param pcs an array of transient instances
     * @exception JDOUserException if an object cannot be made persistent.
     * @see #makePersistent(Object pc)
     */
    public void makePersistentAll(Object[] pcs) {
        if (pcs == null || pcs.length == 0)
            return;
        Exception[] th = new Exception[pcs.length];
        int lg = 0;
        for (int i = 0; i < pcs.length; i++) {
            try {
                makePersistent(pcs[i]);
            } catch (Exception e) {
                th[lg] = e;
                lg++;
            }
        }
        if (lg > 0) {
            Exception[] tfin = new Exception[lg];
            System.arraycopy(th, 0, tfin, 0, lg);
            throw new JDOUserException("Impossible to make persistent : ", tfin);
        }
    }

    /** Make a Set of instances persistent.
     * @param pcs a Set of transient instances
     * @see #makePersistent(Object pc)
     */
    public void makePersistentAll(Collection pcs) {
        if (pcs == null)
            return;
        makePersistentAll(pcs.toArray());
    }

    /** Delete the persistent instance from the data store.
     * This method must be called in an active transaction.
     * The data store object will be removed at commit.
     * Unlike makePersistent, which makes the closure of the instance
     * persistent, the closure of the instance is not deleted from the data
     * store.
     * This method has no effect if the instance is already deleted in the
     * current transaction.
     * @exception JDOUserException if the instance is transient or managed by
     * another PersistenceManager.
     * @param o a persistent instance
     */
    public void deletePersistent(Object o) {
        if (!(o instanceof SpeedoProxy)) {
            throw new JDOException("Impossible to delete a non JDO object " +
                    "or a second class objet:" + o);
        }
        SpeedoProxy sp = (SpeedoProxy) o;
        //the pm must be open
        assertIsOpen();
        //the object must be persistence capable
        assertIsSpeedoProxy(sp);
        //the instance must be managed by this PM
        assertPersistenceManager(sp);
        //we delete the instance
        if (!sp.jdoIsPersistent())
            throw new JDOUserException(
                    "DeletePersistent on an instance non persistent.");
        sp.jdoDeletePersistent(this);
        try {
            tpm.readIntention(tx, sp);
            tpm.writeIntention(tx, sp);
            tpm.unexport(tx, sp);
            sp.jdoChangeStatus(LifeCycle.ACTION_DELETEPERSISTENT);
        } catch (PersistenceException e) {
            throw new JDOException("", ExceptionHelper.getNested(e));
        }
        logger.log(BasicLevel.DEBUG, "Delete the persistent object: "
                + sp + " / id: " + sp.getPName());
    }

    /** Delete an array of instances from the data store.
     * @param pcs a Set of persistent instances
     * @see #deletePersistent(Object pc)
     */
    public void deletePersistentAll(Object[] pcs) {
        if (pcs == null)
            return;
        Throwable[] th = new Throwable[pcs.length];
        int lg = 0;
        for (int i = 0; i < pcs.length; i++) {
            try {
                deletePersistent(pcs[i]);
            } catch (Throwable e) {
                th[lg] = e;
                lg++;
            }
        }
        if (lg > 0) {
            Throwable[] tfin = new Throwable[lg];
            System.arraycopy(th, 0, tfin, 0, lg);
            throw new JDOUserException
                    ("Impossible to delete persistent : ", tfin);
        }
    }

    /** Delete a Set of instances from the data store.
     * @param pcs a Set of persistent instances
     * @see #deletePersistent(Object pc)
     */
    public void deletePersistentAll(Collection pcs) {
        if (pcs == null)
            return;
        deletePersistentAll(pcs.toArray());
    }

    /** Make an instance transient, removing it from management by this
     * PersistenceManager.
     * <P>The instance loses its JDO identity and it is no longer associated
     * with any PersistenceManager.  The state of fields is preserved unchanged.
     * @param o the instance to make transient.
     * @exception JDOUserException if the instance is dirty.
     */
    public void makeTransient(Object o) {
        SpeedoProxy sp = (SpeedoProxy) o;
        assertIsOpen();
        assertIsSpeedoProxy(sp);
        assertPersistenceManager(sp);
        if (!sp.jdoIsPersistent())
            return;
        if (tx.isActive()) {
            if (sp.jdoIsDirty()) {
                throw new JDOUserException(
                        "Try to make transient a dirty instance in a transaction.");
            } else {
                tx.unbind(sp.getPName());
                if (tx.getOptimistic()) {
                    // Is the last SpeedoAccessor
                } else {
                    //There is only one instance
                    desactiveProxy(sp);
                }
            }
        } else {
            if (sp.jdoIsDirty()) {
                throw new JDOUserException("Make transient a dirty instance " +
                        "outside a transaction is not yet supported");
            }
            tx.unbind(sp.getPName());
            desactiveProxy(sp);
        }
    }

    /** Make an array of instances transient, removing them from management by
     * this PersistenceManager.
     * @see #makeTransient(Object pc)
     * @param pcs the instances to make transient.
     */
    public void makeTransientAll(Object[] pcs) {
        if (pcs == null)
            return;
        Throwable[] th = new Throwable[pcs.length];
        int lg = 0;
        for (int i = 0; i < pcs.length; i++) {
            try {
                makeTransient(pcs[i]);
            } catch (Throwable e) {
                th[lg] = e;
                lg++;
            }
        }
        if (lg > 0) {
            Throwable[] tfin = new Throwable[lg];
            System.arraycopy(th, 0, tfin, 0, lg);
            throw new JDOUserException("Impossible to make transient: ", tfin);
        }
    }

    /** Make a Set of instances transient, removing them from management
     * by this PersistenceManager.
     * <P>The instances lose their JDO identity and they are no longer
     * associated with any PersistenceManager.  The state of fields is preserved
     * unchanged.
     * @param pcs the instances to make transient.
     */
    public void makeTransientAll(Collection pcs) {
        if (pcs == null)
            return;
        makeTransientAll(pcs.toArray());
    }

    /** Make an instance subject to transactional boundaries.
     * @see javax.jdo.PersistenceManager#makeTransactional
     * @param o the instance to make transactional.
     */
    public void makeTransactional(Object o) {
        SpeedoProxy sp = (SpeedoProxy) o;
        //preconditions
        assertIsOpen();
        assertIsSpeedoProxy(sp);
        assertPersistenceManager(sp);
        if (!sp.jdoIsPersistent())
            throw new JDOUnsupportedOptionException("not implemented");
//TODO: support the makeTransactional method
//		sp.jdoChangeStatus(LifeCycle.ACTION_MAKETRANSACTIONAL);
        throw new JDOUnsupportedOptionException("not yet implemented");
    }

    /** Make an array of instances subject to transactional boundaries.
     * @param pcs the array of instances to make transactional.
     * @see #makeTransactional(Object pc)
     */
    public void makeTransactionalAll(Object[] pcs) {
        if (pcs == null || pcs.length == 0)
            return;
        Throwable[] th = new Throwable[pcs.length];
        int lg = 0;
        for (int i = 0; i < pcs.length; i++) {
            try {
                makeTransactional(pcs[i]);
            } catch (Throwable e) {
                th[lg] = e;
                lg++;
            }
        }
        if (lg > 0) {
            Throwable[] tfin = new Throwable[lg];
            System.arraycopy(th, 0, tfin, 0, lg);
            throw new JDOUserException
                    ("Impossible to make transactional : ", tfin);
        }
    }

    /** Make a Set of instances subject to transactional boundaries.
     * @param pcs the Set of instances to make transactional.
     * @see #makeTransactional(Object pc)
     */
    public void makeTransactionalAll(Collection pcs) {
        if (pcs == null || pcs.size() == 0)
            return;
        makeTransactionalAll(pcs.toArray());
    }

    public void makeNontransactional(Object o) {
        SpeedoProxy pc = (SpeedoProxy) o;
        //preconditions
        assertIsOpen();
        assertIsSpeedoProxy(pc);
        assertPersistenceManager(pc);
        if (pc.jdoIsDirty())
            throw new JDOUserException("Try to make non-transactional a dirty instance.");
        pc.jdoChangeStatus(LifeCycle.ACTION_MAKENONTRANSACTIONAL);
    }

    public void makeNontransactionalAll(Object[] pcs) {
        if (pcs == null || pcs.length == 0)
            return;
        ArrayList th = new ArrayList(pcs.length);
        for (int i = 0; i < pcs.length; i++) {
            try {
                makeNontransactional(pcs[i]);
            } catch (Throwable e) {
                th.add(e);
            }
        }
        if (th.size() > 0) {
            throw new JDOUserException("Impossible to make non transactional : ",
                    (Throwable[]) th.toArray(new Throwable[0]));
        }
    }

    public void makeNontransactionalAll(Collection pcs) {
        if (pcs == null || pcs.size() == 0)
            return;
        makeNontransactionalAll(pcs.toArray());
    }

    public void setUserObject(Object o) {
        userObject = o;
    }

    /** The application can manage the PersistenceManager instances
     * more easily by having an application object associated with each
     * PersistenceManager instance.
     * @return the user object associated with this PersistenceManager
     * @see #setUserObject
     */
    public Object getUserObject() {
        return userObject;
    }

    public PersistenceManagerFactory getPersistenceManagerFactory() {
        if (_pmf == null) {
            _pmf = (PersistenceManagerFactory)
                    Fractal.getBindingController(((InterfaceReference) pmf)
                    .getFcItfOwner()).lookupFc("persistence-manager-factory");
        }
        return _pmf;
    }


    /** Set the Multithreaded flag for this PersistenceManager.  Applications
     * that use multiple threads to invoke methods or access fields from
     * instances managed by this PersistenceManager must set this flag to true.
     * Instances managed by this PersistenceManager include persistent or
     * transactional instances of PersistenceCapable classes, as well as
     * helper instances such as Query, Transaction, or Extent.
     * @param flag the Multithreaded setting.
     */
    public void setMultithreaded(boolean flag) {
// TODO: take in account this flag
// TODO: thorws JDOUserException if a transaction is runing
        multithreaded = flag;
    }

    /** Get the current Multithreaded flag for this PersistenceManager.
     * This option is ignored by the implementation.
     * @see #setMultithreaded
     * @return the Multithreaded setting.
     */
    public boolean getMultithreaded() {
        return multithreaded;
    }

    /** Set the ignoreCache parameter for queries.
     * @param flag the ignoreCache setting.
     */
    public void setIgnoreCache(boolean flag) {
// TODO: take in account this flag
        ignoreCache = flag;
    }

    /** Get the ignoreCache setting for queries.
     * @return the ignoreCache setting.
     * @see #setIgnoreCache
     */
    public boolean getIgnoreCache() {
        return ignoreCache;
    }

    public Object newObjectIdInstance(Class aClass, String s) {
        try {
            return jf.decode(s);
        } catch (PException e) {
            throw new JDOUserException("Invalid persistent object identifier"
                    + s + " for the class " + aClass);
        }
    }

    public void retrieve(Object o) {
        assertIsOpen();
        assertIsSpeedoProxy(o);
        SpeedoProxy sp = (SpeedoProxy) o;
        assertPersistenceManager(sp);
//TODO: implement the retrieve(Object) mehtod: (proxy loading, proxy state)
    }

    public void retrieveAll(Collection pcs) {
        assertIsOpen();
        if (pcs == null || pcs.size() == 0)
            return;
        ArrayList th = new ArrayList(pcs.size());
        for (Iterator it = pcs.iterator(); it.hasNext();) {
            try {
                makeNontransactional(it.next());
            } catch (Throwable e) {
                th.add(e);
            }
        }
        if (th.size() > 0) {
            throw new JDOUserException("Impossible to make non transactional : ",
                    (Throwable[]) th.toArray(new Throwable[0]));
        }
    }

    public void retrieveAll(Object[] pcs) {
        assertIsOpen();
        if (pcs == null || pcs.length == 0)
            return;
        ArrayList th = new ArrayList(pcs.length);
        for (int i = 0; i < pcs.length; i++) {
            try {
                makeNontransactional(pcs[i]);
            } catch (Throwable e) {
                th.add(e);
            }
        }
        if (th.size() > 0) {
            throw new JDOUserException("Impossible to retrieve all object: ",
                    (Throwable[]) th.toArray(new Throwable[0]));
        }
    }


    // IMPLEMENTATION OF THE ProxyManager INTERFACE //
    //----------------------------------------------//

    public TransactionalPersistenceManager getTransactionalPersistenceManager() {
        return tpm;
    }

    /**
     * Sets the reference between two JDO instances
     *
     * @param sp the reference object whose proxy infrastructure will be build
     * @param father the owner
     * @param name is the name of the field
     */
    public PName createFieldOfProxy(SpeedoProxy sp,
                                    SpeedoProxy father,
                                    String name) throws Exception {
        if (logger.isLoggable(BasicLevel.DEBUG)) {
            logger.log(BasicLevel.DEBUG, "proxy=" + sp);
            logger.log(BasicLevel.DEBUG, "father=" + father);
            logger.log(BasicLevel.DEBUG, "name=" + name);
        }
        if (!sp.jdoIsActive()) {
            if (sp instanceof SpeedoGenClassProxy) {
                ((SpeedoGenClassProxy) sp).jdoSetPBinding(getGenClassBinding((SpeedoGenClassProxy) sp));
                if (sp instanceof Loggable) {
                    ((Loggable) sp).setLogger(logger);
                }
            }
            sp.init(jf.getPClassMapping(sp));
            sp.jdoSetTransactionalPersistenceManager(tpm);
            sp.jdoSetPersistenceManagerFactory(getPersistenceManagerFactory());
            sp.jdoSetPersistenceManagerSwitch(pms);
            tpm.readIntention(tx, sp);
            sp.jdoIsActive(true);
        }
        return sp.getPName();
    }

    public void open(Object connectionspec) {
        connectionSpec = connectionspec;
        isClosed = false;
        tx.setOptimistic(pmf.getOptimistic());
        try {
            tx.active();
        } catch (PersistenceException e) {
            throw new JDOFatalInternalException(
                    "Impossible to open this persistence manager", e);
        }
    }

    public Object getConnectionSpec() {
        return connectionSpec;
    }
    // OTHER METHODS //
    //---------------//


    /**
     * Removes the proxy infrastructure of a JDO instance
     *
     * @param sp the object requiring architecture deletion
     */
    private void desactiveProxy(SpeedoProxy sp) {
        try {
            sp.init(null);
        } catch (PException e) {
            throw new JDOException(
                    "Impossible to unassign the PClassMapping to the proxy", e);
        }
        sp.jdoSetTransactionalPersistenceManager(null);
        sp.jdoSetPersistenceManagerFactory(null);
        sp.jdoSetPersistenceManagerSwitch(null);
        sp.jdoIsActive(false);
        if (sp instanceof SpeedoGenClassProxy) {
            ((SpeedoGenClassProxy) sp).jdoSetPBinding(null);
            if (sp instanceof Loggable) {
                ((Loggable) sp).setLogger(logger);
            }
        }
        try {
            replacementManager.unbind(sp);
        } catch (CacheException e) {
            throw new JDOException(
                    "Impossible to remove the persistent object from the cache", e);
        }
    }

    /**
     * Add the proxy infrastructure of a JDO instance
     * The assignement of the PClassMapping must be done outside of this method.
     *
     * @param sp the object requiring architecture deletion
     */
    private void activeProxy(SpeedoProxy sp) {
        if (sp instanceof SpeedoGenClassProxy) {
            ((SpeedoGenClassProxy) sp).jdoSetPBinding(getGenClassBinding((SpeedoGenClassProxy) sp));
            if (sp instanceof Loggable) {
                ((Loggable) sp).setLogger(logger);
            }
        }
        if (sp.getPClassMapping() == null) {
            try {
                sp.init(jf.getPClassMapping(sp));
            } catch (PException e) {
            }
        }
        sp.jdoSetPersistenceManagerSwitch(pms);
        sp.jdoSetTransactionalPersistenceManager(tpm);
        sp.jdoSetPersistenceManagerFactory(getPersistenceManagerFactory());
        sp.jdoIsActive(true);
        if (logger.isLoggable(BasicLevel.DEBUG))
            logger.log(BasicLevel.DEBUG, "Proxy activated: " + sp);
    }


    /**
     * Reload an object from the data base.
     * @param pc object to reload.
     * @exception JDODataStoreException if there is a problem while loading the object.
     */
    private void
        readObject(SpeedoProxy pc) {
        try {
            synchronized (pc) {
                SpeedoAccessor pa = pc.getSpeedoAccessor();
                if (!pc.jdoIsNew()) {
                    logger.log(BasicLevel.DEBUG, "read");
                    Object conn;
                    if (connectionSpec == null) {
                        conn = mapper.getConnection();
                    } else {
                        conn = mapper.getConnection(connectionSpec);
                    }
                    pc.read(conn, pa);
                    mapper.closeConnection(conn);
                    if (pc instanceof InstanceCallbacks)
                        ((InstanceCallbacks) pc).jdoPostLoad();
                }
            }
        } catch (Exception e) {
            throw new JDODataStoreException("Problem while loading the value.",
                    new Exception[]{e});
        }
    }

    /**
     * Verify the persistence manager is open.
     * @exception javax.jdo.JDOFatalUserException if the persistence manager is
     * closed.
     */
    private void assertIsOpen() {
        if (isClosed)
            throw new JDOFatalUserException("The persistent manager is closed.");
    }

    /**
     * Verify that an instance is persistence capable.
     * @param pc the object to test.
     * @exception javax.jdo.JDOUserException if the object is not persistence capable.
     */
    private void assertIsSpeedoProxy(Object pc) {
        if (!(pc instanceof SpeedoProxy))
            throw new JDOUserException("The object is not a Speedoproxy: " + pc);
    }

    /**
     * Verify the instance is managed by this persistence manager.
     * @param sp a speedo proxy instance.
     * @exception javax.jdo.JDOUserException if the instance is managed by
     * another persistence manager.
     */
    private void assertPersistenceManager(SpeedoProxy sp) {
        if (!getThisPM().equals(sp.jdoGetPersistenceManager())) {
            ProxyManager pm = (ProxyManager) sp.jdoGetPersistenceManager();
            if (pm == null) {
                try {
                    tpm.readIntention(tx, sp);
                } catch (PersistenceException e) {
                    throw new JDOUserException("Impossible to add the " +
                            "persistent object into the working set", e);
                }
            } else {
                throw new JDOUserException(
                    "Object not managed by this persistence manager, object: "
                    + sp);
            }
        }
    }

    private ProxyManager getThisPM() {
        if (!(thisPM instanceof ProxyManager)) {
            SubComponentController parentCtrl = (SubComponentController)
                    ((ComponentIdentity) thisPM).getFcInterface("subcomponent-controller");
            thisPM = parentCtrl.getFcParents()[0].getFcInterface("persistence-manager");
        }
        return (ProxyManager) thisPM;
    }

    private PBinding getGenClassBinding(SpeedoGenClassProxy sp) {
        String mapperName = sp.jdoGetMapperName();
        if (mapperName.startsWith("rdb")) {
            try {
                return (PBinding) Class.forName(
                        "org.objectweb.jorm.interp.genclass.rdb.lib.RdbGenClassBinding"
                ).newInstance();
            } catch (Exception e) {
                throw new JDOFatalInternalException(
                        "Impossible to find a PBinding for the Generic class " + sp, e);
            }
        } else {
            throw new JDOFatalInternalException("Umanaged mapper: " + mapperName);
        }

    }
}
