/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.genclass;

import org.objectweb.speedo.runtime.concurrence.optimistic.BasicSpeedoAccessor;
import org.objectweb.speedo.api.SpeedoGenClassProxy;
import org.objectweb.jorm.api.PGenClassAccessor;
import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.util.monolog.api.Logger;

/**
 * This class represents an accessor of a genclass structure.
 * This accessor is used in case of the generic class is persistent.
 *
 * @author S.Chassande-Barrioz
 */
public abstract class GenClassAccessor
        extends BasicSpeedoAccessor
        implements PGenClassAccessor {

    /**
     * The SpeedoProxy attached to this state representation.
     */
    public GenClass jdoProxy;

    /**
     * The logger in which the traces will be send.
     */
    public Logger logger = null;

    /**
     * Indicates if this accessor support the dela mode
     */
    public boolean supportDelta = true;

    /**
     * A temporary variable which is used to search an element into the
     * "elements" list.
     */
    protected GenClassElement tmpelem = null;


    public GenClassAccessor(GenClass jdoProxy) {
        this.jdoProxy = jdoProxy;
        this.logger = jdoProxy.getLogger();
        this.tmpelem = (GenClassElement) createPIndexedElem();
    }

    // IMPLEMENTATION OF THE PGenClassAccessor INTERFACE //
    //---------------------------------------------------//

    public abstract PIndexedElem createPIndexedElem();

    public Object getMemoryInstance() {
        return jdoProxy;
    }

    public boolean paDeltaSupported() {
        return supportDelta;
    }
}
