/**
 * copyright area
 */
package org.objectweb.speedo.runtime.genclass.map;

import org.objectweb.speedo.runtime.concurrence.optimistic.BasicSpeedoAccessor;
import org.objectweb.speedo.runtime.genclass.GenClassElement;
import org.objectweb.speedo.runtime.genclass.GenClassAccessor;
import org.objectweb.speedo.api.SpeedoProxy;
import org.objectweb.speedo.api.ProxyManager;
import org.objectweb.jorm.api.PGenClassAccessor;
import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.util.monolog.api.Logger;

import java.util.Map;
import java.util.Iterator;
import java.util.Set;
import java.util.Collection;
import java.util.ArrayList;

public class MapAccessor
        extends GenClassAccessor
        implements Map {

    /**
     * The ArrayList used to store the indexed elements of the genclass.
     * Then this list contains also the deleted elements.
     */
    protected Collection elements = null;

    /**
     * A map containing the indexed elements of the genclass. It associates each
     * element to its index. This map does NOT contain the deleted elements.
     */
    protected Map map = null;

    /**
     * Instanciates and initializes a new map with an initial size.
     */
    public MapAccessor(MapImpl jdoProxy, int size) {
        super(jdoProxy);
        this.elements = new ArrayList();
        this.map = jdoProxy.createMap(size);
    }

    // ------------------------------------------------------------------------
    // IMPLEMENTATION OF THE Map INTERFACE
    // ------------------------------------------------------------------------

    public int size() {
        return map.size();
    }

    public boolean isEmpty() {
        return map.isEmpty();
    }

    public boolean containsKey(Object key) {
        return map.containsKey(key);
    }

    public boolean containsValue(Object value) {
        /*tmpelem.setIndex(); // TODO
        tmpelem.setElement(value);
        return map.containsValue(tmpelem);*/
        return false;
    }

    public Object get(Object key) {
        Object value = map.get(key);
        if (value == null) {
            return null;
        } else {
            return ((MapElem) value).getRefElement(
                    jdoProxy.jdoGetPersistenceManager());
        }
    }

    public Object put(Object key, Object value) {
        // Put it in "elements" and "map"
        GenClassElement element = (GenClassElement) createPIndexedElem();
        if (value instanceof SpeedoProxy) {
            ProxyManager pm = (ProxyManager) jdoProxy.jdoGetPersistenceManager();
            jdoProxy.jdoMakePersistent(pm, (SpeedoProxy) value);
        }
        element.setIndex(key);
        element.setElement(value);
        elements.add(element); // TODO et si element est deja dans elements? s'il est marque deleted?
        Object oldValue = get(key);
        map.put(key, element);
        return oldValue;
    }

    public Object remove(Object key) {
        GenClassElement gcelem = (GenClassElement) map.get(key);
        if (gcelem != null) {
            gcelem.setStatus(PIndexedElem.ELEM_DELETED);
        }
        Object value = get(key);
        map.remove(key);
        return value;
    }

    public void putAll(Map t) {
        Iterator i = t.entrySet().iterator();
        while (i.hasNext()) {
            Map.Entry entry = (Map.Entry) i.next();
            put(entry.getKey(), entry.getValue());
        }
    }

    public void clear() {
        Iterator i = elements.iterator();
        while (i.hasNext()) {
            GenClassElement gcelem = (GenClassElement) i.next();
            gcelem.setStatus(PIndexedElem.ELEM_DELETED);
        }
        map.clear();
    }

    public Set keySet() {
        return null; // TODO
    }

    public Collection values() {
        return null; // TODO
    }

    public Set entrySet() {
        return null; // TODO
    }

    // ------------------------------------------------------------------------
    // IMPLEMENTATION OF THE PGenClassAccessor INTERFACE
    // ------------------------------------------------------------------------

    public PIndexedElem createPIndexedElem() {
        return new MapElem();
    }

    public void paAdd(PIndexedElem elem, Object conn) throws PExceptionIO {
        GenClassElement gcelem = (GenClassElement) elem;
        // the elem is read from the DS, set it to unmodified.
        gcelem.setStatus(PIndexedElem.ELEM_UNMODIFIED);
        elements.add(gcelem);
        map.put(gcelem.getIndex(), gcelem);
    }

    public int paGetNbElem() {
        return size();
    }

    public Iterator paIterator() {
        return elements.iterator();
    }

    public void paSetNbElem(int nbelem) {
        elements = new ArrayList(nbelem);
        map = ((MapImpl) jdoProxy).createMap(nbelem);
    }
}
