/**
 * copyright area
 */
package org.objectweb.speedo.runtime.genclass.map;

import org.objectweb.speedo.runtime.genclass.GenClass;
import org.objectweb.speedo.api.SpeedoAccessor;
import org.objectweb.speedo.api.ProxyManager;
import org.objectweb.speedo.api.SpeedoProxy;
import org.objectweb.speedo.api.SpeedoGenClassProxy;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.naming.api.PName;

import java.util.Map;
import java.util.Collection;
import java.util.Set;
import java.util.HashMap;
import java.util.Iterator;

public class MapImpl extends GenClass implements Map {

    /**
     * is the internal map used when the generic class is not persistent
     */
    protected Map map = null;

    /**
     * Instantiates a new map.
     */
    public MapImpl() {
        this(10);
    }

    /**
     * Instanciates and initializes a new map with an initial size.
     */
    public MapImpl(int size) {
        super();
        map = createMap(size);
    }
    public MapImpl(Map map,
                   PType ptype,
                   String linkedField,
                   PName linkedPName,
                   String mapperName) {
        super(ptype, linkedField, linkedPName, mapperName);
        setInner(map);
    }

    public MapImpl(PType ptype,
                          String linkedField,
                          PName linkedPName,
                          String mapperName) {
        super(ptype, linkedField, linkedPName, mapperName);
    }


    // ------------------------------------------------------------------------
    // IMPLEMENTATION OF THE Collection INTERFACE
    // ------------------------------------------------------------------------

    public int size() {
        if (!jdoIsActive) {
            return map.size();
        } else {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ca);
            return ca.size();
        }
    }

    public boolean isEmpty() {
        if (!jdoIsActive) {
            return map.isEmpty();
        } else {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ca);
            return ca.isEmpty();
        }
    }

    public boolean containsKey(Object key) {
        if (!jdoIsActive) {
            return map.containsKey(key);
        } else {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ca);
            return ca.containsKey(key);
        }
    }

    public boolean containsValue(Object value) {
        if (!jdoIsActive) {
            return map.containsValue(value);
        } else {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ca);
            return ca.containsValue(value);
        }
    }

    public Object get(Object key) {
        if (!jdoIsActive) {
            return map.get(key);
        } else {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ca);
            return ca.get(key);
        }
    }

    public Object put(Object key, Object value) {
        if (!jdoIsActive) {
            return map.put(key, value);
        } else {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoWriteIntention(ca);
            return ca.put(key, value);
        }
    }

    public Object remove(Object key) {
        if (!jdoIsActive) {
            return map.remove(key);
        } else {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoWriteIntention(ca);
            return ca.remove(key);
        }
    }

    public void putAll(Map t) {
        if (!jdoIsActive) {
            map.putAll(t);
        } else {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoWriteIntention(ca);
            ca.putAll(t);
        }
    }

    public void clear() {
        if (!jdoIsActive) {
            map.clear();
        } else {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoWriteIntention(ca);
            ca.clear();
        }
    }

    public Set keySet() {
        if (!jdoIsActive) {
            return map.keySet();
        } else {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ca);
            return ca.keySet();
        }
    }

    public Collection values() {
        if (!jdoIsActive) {
            return map.values();
        } else {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ca);
            return ca.values();
        }
    }

    public Set entrySet() {
        if (!jdoIsActive) {
            return map.entrySet();
        } else {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ca);
            return ca.entrySet();
        }
    }

    // IMPLEMENTATION OF THE SpeedoProxy INTERFACE //
    //---------------------------------------------//

    public void setGCValue(Object o) {
        if (jdoIsActive) {
            MapAccessor ca = (MapAccessor) getSpeedoAccessor();
            jdoWriteIntention(ca);
            ca.clear();
            if (o != null) {
                ca.putAll(((Map) o));
            }
        } else {
            map.clear();
            if (o != null) {
                map.putAll(((Map) o));
            }
        }
    }

    protected SpeedoAccessor createAccessor() {
        return new MapAccessor(this, map.size());
    }

    public void jdoLoadFieldsInAccessor(SpeedoAccessor sa) {
        MapAccessor ma = (MapAccessor) sa;
        ma.clear();
        ma.putAll(map);
    }

    public void jdoLoadFieldsFromAccessor(SpeedoAccessor sa) {
        map.clear();
        map.putAll((MapAccessor) sa);
    }

    public Collection getGenClassElements(SpeedoAccessor sa) {
        return ((MapAccessor) sa).elements;
    }

    public void jdoMakePersistent(ProxyManager pm) {
        if (map.size() == 0) {
            return;
        }
        Iterator i = map.values().iterator();
        while (i.hasNext()) {
            Object elem = i.next();
            if (elem instanceof SpeedoProxy) {
                jdoMakePersistent(pm, (SpeedoProxy) elem);
            } else {
                // Optimisation: If the first element is not a SpeedoProxy, that
                // means that ALL map elements are not SpeedoProxy
                // implementation
                return;
            }
        }
    }

    // OTHER METHODS //
    // --------------//

    protected Map createMap(int size) {
        return new HashMap(size);
    }

    protected MapImpl createInstance(int size) {
        return new MapImpl(size);
    }
}
