/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: X. Spengler
 *
 */

package org.objectweb.speedo.runtime.query;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PException;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

import javax.jdo.JDOUnsupportedOptionException;
import javax.jdo.PersistenceManager;
import javax.jdo.Transaction;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

public class QueryResultCollection implements Collection {

    private TupleCollection tc = null;
    private PersistenceManager pm = null;
    private List values = null;
    private Object[] conns = null;
    private PMapper mapper = null;
    private int pnIndex;
    private Logger logger;

    public void init(TupleCollection tc,
                     PersistenceManager pm,
                     Object[] conns,
                     PMapper mapper,
                     Logger logger) throws MedorException {
        this.tc = tc;
        this.pm = pm;
        this.conns = conns;
        this.mapper = mapper;
        this.pnIndex = tc.getMetaData().getSize();
        this.logger = logger;
        logger.log(BasicLevel.DEBUG, "index of the pname: " + pnIndex);
    }

    public void close() {
        pm = null;
        for (int i=0; i<conns.length; i++) {
            try {
                mapper.closeConnection(conns[i]);
            } catch (PException e) {
                e.printStackTrace();
            }
        }
        conns = null;
        mapper = null;
    }

    /**
     * Only add the PName to the arraylist
     */
    private List getList() {
        if (values != null)
            return values;

        values = new ArrayList();
        try {
            if (!tc.isEmpty()) {
                tc.first();
                do {
                    values.add(pm.getObjectById(
                        tc.getTuple().getObject(pnIndex), false));
                } while (tc.next());
            }
        } catch (MedorException me) {
            throw new JDOUnsupportedOptionException(
                    "Persistence manager problem to get result", new Exception[]{me});
        }
        return values;
    }

    private class QueryIterator implements Iterator {

        private boolean hasNext;

        public QueryIterator() {
            try {
                tc.first();
                hasNext = !tc.isEmpty() && pm != null && !pm.isClosed();
            } catch (MedorException e) {
                hasNext = false;
            }
        }

        public boolean hasNext() {
            return hasNext && pm != null && !pm.isClosed();
        }

        public Object next() {
            if (hasNext()) {
                Object res;
                try {
                    res = pm.getObjectById(
                        tc.getTuple().getObject(pnIndex), false);
                    hasNext = tc.next();
                } catch (MedorException e) {
                    logger.log(BasicLevel.ERROR, "Impossible to fetch the pname", e);
                    throw new NoSuchElementException(e.getMessage());
                }
                return res;
            } else
                throw new NoSuchElementException("No such element");
        }

        public void remove() {
            throw new UnsupportedOperationException(
                    "This method is not supported in this implementation");
        }
    }


    // IMPLEMENTATION OF THE Collection INTERFACE //
    //--------------------------------------------//

    /**
     * Ensures that this collection contains the specified element
     * (optional operation).
     */
    public boolean add(Object o) {
        throw new JDOUnsupportedOptionException(
                "This method is not supported in this implementation");
    }

    /**
     * Adds all of the elements in the specified collection to this collection
     * (optional operation).
     */
    public boolean addAll(Collection c) {
        throw new JDOUnsupportedOptionException(
                "This method is not supported in this implementation");
    }

    /**
     * Removes all of the elements from this collection
     * (optional operation).
     */
    public void clear() {
        throw new JDOUnsupportedOptionException(
                "This method is not supported in this implementation");
    }

    /**
     * Returns true if this collection contains the specified element.
     */
    public boolean contains(Object o) {
        return getList().contains(o);
    }

    /**
     * Returns true if this collection contains all of the elements in the
     * specified collection.
     */
    public boolean containsAll(Collection c) {
        getList();
        for (Iterator iter = c.iterator(); iter.hasNext();) {
            if (!values.contains(iter.next()))
                return false;
        }
        return true;
    }

    /**
     * Compares the specified object with this collection for equality.
     */
    public boolean equals(Object o) {
        throw new JDOUnsupportedOptionException(
                "This method is not supported in this implementation");
    }

    /**
     * Returns the hash code value for this collection.
     */
    public int hashCode() {
        return getList().hashCode();
    }

    /**
     * Returns true if this collection contains no elements.
     */
    public boolean isEmpty() {
        try {
            return tc == null || tc.isEmpty();
        } catch (MedorException me) {
            throw new JDOUnsupportedOptionException(
                    "a MedorException has been catched",
                    new Exception[]{me});
        }
    }

    /**
     * Returns an iterator over the elements in this collection.
     */
    public Iterator iterator() {
        getList();
        return new QueryIterator();
    }

    /**
     * Removes a single instance of the specified element from this collection,
     * if it is present (optional operation).
     */
    public boolean remove(Object o) {
        throw new JDOUnsupportedOptionException(
                "This method is not supported in this implementation");
    }

    /**
     * Removes all this collection's elements that are also contained in the
     * specified collection (optional operation).
     */
    public boolean removeAll(Collection c) {
        throw new JDOUnsupportedOptionException(
                "This method is not supported in this implementation");
    }

    /**
     * Retains only the elements in this collection that are contained in the
     * specified collection (optional operation).
     */
    public boolean retainAll(Collection c) {
        throw new JDOUnsupportedOptionException(
                "This method is not supported in this implementation");
    }

    /**
     * Returns the number of elements in this collection.
     */
    public int size() {
        return getList().size();
    }

    /**
     * Returns an array containing all of the elements in this collection.
     */
    public Object[] toArray() {
        return getList().toArray();
    }

    /**
     * Returns an array containing all of the elements in this collection whose
     * runtime type is that of the specified array.
     */
    public Object[] toArray(Object[] a) {
        return getList().toArray(a);
    }
}
