/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: P. Dechamboux, Y. Devaux, F. Dhumeaux.
 *
 */

package org.objectweb.speedo.runtime.query;

import java.util.Iterator;

import javax.jdo.Extent;
import javax.jdo.JDOUnsupportedOptionException;
import javax.jdo.PersistenceManager;

/**
 */
public class SpeedoExtent implements Extent {

    private Class candidateClass = null;
    private boolean hasSubclasses = false;
    private PersistenceManager pm = null;

    /**
     * create a new SpeedoExtent object, this object is not obtained by a
     * call to 'new' by the client application, but by the PersistenceManager
     * object with getExtent() method
     * @param candidateClass
     * @param hasSubclasses
     * @param pm
     */
    public SpeedoExtent(Class candidateClass,
                        boolean hasSubclasses,
                        PersistenceManager pm) {

        this.candidateClass = candidateClass;
        this.hasSubclasses = hasSubclasses;
        this.pm = pm;
    }

    /**
     * Returns an iterator over all the instances in the Extent.
     * @return an iterator over all instances in the Extent
     */
    public Iterator iterator() {
        throw new JDOUnsupportedOptionException("not implemented");
    }

    /**
     * Returns whether this Extent was defined to contain subclasses.
     * @return true if this Extent was defined to contain instances
     * that are of a subclass type
     */
    public boolean hasSubclasses() {
        return hasSubclasses;
    }

    /**
     * An Extent contains all instances of a particular Class in the data
     * store; this method returns the Class of the instances
     * @return the Class of instances of this Extent
     */
    public Class getCandidateClass() {
        return candidateClass;
    }

    /**
     * An Extent is managed by a PersistenceManager; this method gives access
     * to the owning PersistenceManager.
     * @return the owning PersistenceManager
     */
    public PersistenceManager getPersistenceManager() {
        return pm;
    }

    /**
     * Close all Iterators associated with this Extent instance.  Iterators closed
     * by this method will return false to hasNext() and will throw
     * NoSuchElementException on next().  The Extent instance can still be used
     * as a parameter of Query.setExtent, and to get an Iterator.
     */
    public void closeAll() {
        throw new JDOUnsupportedOptionException("not implemented");
    }

    /**
     * Close an Iterator associated with this Extent instance.  Iterators closed
     * by this method will return false to hasNext() and will throw
     * NoSuchElementException on next().  The Extent instance can still be used
     * as a parameter of Query.setExtent, and to get an Iterator.
     * @param it an iterator obtained by the method iterator() on this Extent instance.
     */
    public void close(Iterator it) {
        throw new JDOUnsupportedOptionException("not implemented");
    }
}
