/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: X. Spengler
 *
 */

package org.objectweb.speedo.runtime.query;

import org.objectweb.fractal.api.control.UserBindingController;
import org.objectweb.speedo.runtime.query.api.QueryCompiler;
import org.objectweb.speedo.runtime.query.api.QueryDefinition;
import org.objectweb.speedo.runtime.query.api.QueryManager;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.JormFactory;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.perseus.pool.api.Pool;
import org.objectweb.perseus.pool.api.PoolMatchFactory;
import org.objectweb.perseus.pool.api.PoolResource;
import org.objectweb.medor.lib.Log;

import javax.jdo.JDOException;
import java.util.HashMap;
import java.util.Map;

/**
 * SpeedoQueryManager manages the association between SpeedoQuery and the
 * the compiled query: SpeedoQueryCompiler.
 *
 * @author X.Spengler, S.Chassande-Barrioz
 */
public class SpeedoQueryManager
        implements QueryManager, PoolMatchFactory, UserBindingController {

    public final static String MAPPER_BINDING = "mapper";
    public final static String JORM_FACTORY_BINDING = "jorm-factory";
    public final static String LOGGER_FACTORY_BINDING = "logger-factory";
    public final static String QUERY_POOL_BINDING = "query-compiler-pool";
    public final static String LOGGER_NAME
            = "org.objectweb.speedo.rt.query.SpeedoQueryManager";

    //map between Query and SpeedoQueryCompiler
    private Map map;

    private PMapper mapper = null;

    private JormFactory jormFactory = null;

    private Pool sQueries = null;

    private LoggerFactory loggerFactory = null;

    private Logger logger = null;

    /**
     * creates a new SpeedoQueryManager object.
     */
    public SpeedoQueryManager() {
        map = new HashMap();
    }

    public PoolResource createResource(Object hints) throws Exception {
        SpeedoQueryCompiler qc = new SpeedoQueryCompiler();
        if (hints != null)
            qc.defineQuery((QueryDefinition) hints);
        qc.setMapper(mapper);
        qc.setLoggerFactory(loggerFactory);
        qc.setJormFactory(jormFactory);
        logger.log(BasicLevel.DEBUG,
            "create a new SpeedoQuery (hints=" + hints + ")");
        return qc;
    }

    public boolean matchResource(PoolResource pr, Object hints) {
        try {
            if (hints != null)
                ((SpeedoQueryCompiler) pr).defineQuery((QueryDefinition) hints);
            return true;
        } catch (SpeedoException e) {
            throw new JDOException(
                    "Impossible to redefine compiled query coming from the pool", e);
        }
    }

    // IMPLEMENTATION OF UserBindingController INTERFACE //
    //---------------------------------------------------//

    public Object getFcBindings(String s) {
        if (LOGGER_FACTORY_BINDING.equals(s))
            return loggerFactory;
        else if (MAPPER_BINDING.equals(s))
            return mapper;
        else if (JORM_FACTORY_BINDING.equals(s))
            return jormFactory;
        else if (QUERY_POOL_BINDING.equals(s))
            return sQueries;
        else
            return null;
    }

    public void addFcBinding(String s, Object o) {
        if (LOGGER_FACTORY_BINDING.equals(s)) {
            loggerFactory = (LoggerFactory) o;
            logger = loggerFactory.getLogger(LOGGER_NAME);
            Log.loggerFactory = loggerFactory;
        } else if (MAPPER_BINDING.equals(s))
            mapper = (PMapper) o;
        else if (QUERY_POOL_BINDING.equals(s))
            sQueries = (Pool) o;
        else if (JORM_FACTORY_BINDING.equals(s))
            jormFactory = (JormFactory) o;
    }

    public void removeFcBinding(String s, Object serverItf) {
        if (LOGGER_FACTORY_BINDING.equals(s))
            loggerFactory = null;
        else if (MAPPER_BINDING.equals(s))
            mapper = null;
        else if (JORM_FACTORY_BINDING.equals(s))
            jormFactory = null;
    }


    // IMPLEMENTATION OF QueryManager INTERFACE //
    //------------------------------------------//

    /**
     * returns a QueryCompiler implementation, creates it if it does not exists,
     * or just returns an existing one.
     * @param qd	a QueryDefinition
     * @return	a (new/existing) QueryCompiler instance.
     */
    public QueryCompiler getQueryCompiler(QueryDefinition qd) {

        // compute the string which will be compared to existing string in
        // the HashMap (existing querys)
        String internal = qd.getCandidateClass().getName();

        if (qd.isCollection())
            internal += " " + qd.getCollection().hashCode();
        else
            internal += " " + "Extent";

        internal += "@" + qd.getParameters();
        internal += "@" + qd.getVariables();
        internal += "@" + qd.getFilter();
        internal += "@" + qd.getOrder();

        SpeedoQueryCompiler qc = (SpeedoQueryCompiler) map.get(internal);
        if (qc == null) {
            try {
                qc = (SpeedoQueryCompiler) sQueries.getResource(qd);
            } catch (JDOException e) {
                throw e;
            } catch (Exception e) {
                throw new JDOException(
                        "Impossible to fetch compiled query from the pool", e);
            }
            map.put(internal, qc);
        }
        return qc;
    }
}
