/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: X. Spengler
 *
 */

package org.objectweb.speedo.runtime.query.parser;

import org.objectweb.medor.api.Field;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.lib.And;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.filter.lib.BasicOperand;
import org.objectweb.medor.filter.lib.BasicParameterOperand;
import org.objectweb.medor.filter.lib.Bitwize;
import org.objectweb.medor.filter.lib.ConditionalAnd;
import org.objectweb.medor.filter.lib.ConditionalOr;
import org.objectweb.medor.filter.lib.DivideBy;
import org.objectweb.medor.filter.lib.Equal;
import org.objectweb.medor.filter.lib.Greater;
import org.objectweb.medor.filter.lib.GreaterEqual;
import org.objectweb.medor.filter.lib.Lower;
import org.objectweb.medor.filter.lib.LowerEqual;
import org.objectweb.medor.filter.lib.Minus;
import org.objectweb.medor.filter.lib.Mult;
import org.objectweb.medor.filter.lib.Not;
import org.objectweb.medor.filter.lib.NotEqual;
import org.objectweb.medor.filter.lib.Or;
import org.objectweb.medor.filter.lib.Plus;
import org.objectweb.medor.filter.lib.UMinus;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Stack;

/**
 * Implementation of a visitor that creates the filter
 */
public class SpeedoQLQueryFilterVisitor extends SpeedoQLAbstractVisitor {
    /**
     * qf is the transformation of the filter into a MEDOR expression
     */
    private Expression qf = null;

    /**
     * qt is built by the SpeedoQLVariableVisitor and is just used here
     */
    private QueryTree qt = null;

    /**
     * speedoql is the result of the filter parsing
     */
    private ASTSpeedoQL speedoql = null;

    /**
     * Constructor
     * @param qtree a QueryTree built with the list of variables, parameters and name.
     * @param speedoql root of the lexical tree of the query
     */
    public SpeedoQLQueryFilterVisitor(QueryTree qtree,
                                      ASTSpeedoQL speedoql) {
        qt = qtree;
        this.speedoql = speedoql;
    }

    /**
     * the visit starts here, please before sets a list of variables.
     * @throws Exception
     */
    public void startVisiting() throws Exception {
        visit(speedoql);
    }

    /**
     * get the query filter that was built from visiting the syntaxic tree
     */
    public Expression getQueryFilter() {
        return qf;
    }

    public Object visit(ASTPrimary node, Object data) {
        visit((SimpleNode) node, data);
        return null;
    }

    public Object visit(ASTSpeedoPrimary node, Object data) {
        visit((SimpleNode) node, data);
        qf = (Expression) ((Stack) data).pop();
        return null;
    }

    /**
     *
     */
    public Object visit(ASTRelationalExpression node, Object data) {
        logger.log(BasicLevel.DEBUG, "Visit RelationalExpression");
        visit((SimpleNode) node, data);
        if (((Stack) data).size() > 0) {
            Expression ret = (Expression) ((Stack) data).pop();
            logger.log(BasicLevel.DEBUG, "CHILDREN=[" + node.jjtGetNumChildren() + "] stack size=(" + ((Stack) data).size() + ")");

            for (int i = 0; ((i < node.jjtGetNumChildren() - 1) && (((Stack) data).size() > 0)); i++) {
                logger.log(BasicLevel.DEBUG, "Visit ConditionalExpression... children...[" + i + "]");

                switch (((Integer) node.ops.get(node.jjtGetNumChildren() - 2 - i)).intValue()) {
                case SpeedoQLConstants.OR:
                    ret = new ConditionalOr((Expression) ((Stack) data).pop(), ret);
                    break;
                case SpeedoQLConstants.AND:
                    ret = new ConditionalAnd((Expression) ((Stack) data).pop(), ret);
                    break;
                case SpeedoQLConstants.BITWISEOR:
                    ret = new Or((Expression) ((Stack) data).pop(), ret);
                    break;
                    // en cours par equipe MEDOR
                case SpeedoQLConstants.BITWISEXOR:
                    ret = null;
                    break;
                case SpeedoQLConstants.BITWISEAND:
                    ret = new And((Expression) ((Stack) data).pop(), ret);
                    break;
                case SpeedoQLConstants.EQ:
                    ret = new Equal((Expression) ((Stack) data).pop(), ret);
                    break;
                case SpeedoQLConstants.NEQ:
                    ret = new NotEqual((Expression) ((Stack) data).pop(), ret);
                    break;
                case SpeedoQLConstants.LT:
                    ret = new Lower((Expression) ((Stack) data).pop(), ret);
                    break;
                case SpeedoQLConstants.GT:
                    ret = new Greater((Expression) ((Stack) data).pop(), ret);
                    break;
                case SpeedoQLConstants.GTE:
                    ret = new GreaterEqual((Expression) ((Stack) data).pop(), ret);
                    break;
                case SpeedoQLConstants.LTE:
                    ret = new LowerEqual((Expression) ((Stack) data).pop(), ret);
                    break;
                }
            }

            ((Stack) data).push(ret);
        }
        logger.log(BasicLevel.DEBUG, "End of Visit RelationalExpression");
        return null;
    }

    public Object visit(ASTAdditiveExpression node, Object data) {
        logger.log(BasicLevel.DEBUG, "Visit AdditiveExpression");
        visit((SimpleNode) node, data);
        if (((Stack) data).size() > 0) {
            Expression ret = (Expression) ((Stack) data).pop();

            for (int i = 0; i < node.jjtGetNumChildren() - 1; i++) {
                logger.log(BasicLevel.DEBUG, "Visit ConditionalExpression... children...[" + i + "]");

                switch (((Integer) node.ops.get(node.jjtGetNumChildren() - 2 - i)).intValue()) {

                case SpeedoQLConstants.PLUS:
                    ret = new Plus((Expression) ((Stack) data).pop(), ret);
                    break;
                case SpeedoQLConstants.MINUS:
                    ret = new Minus((Expression) ((Stack) data).pop(), ret);
                    break;
                case SpeedoQLConstants.MULT:
                    ret = new Mult((Expression) ((Stack) data).pop(), ret);
                    break;
                case SpeedoQLConstants.DIV:
                    ret = new DivideBy((Expression) ((Stack) data).pop(), ret);
                    break;
                }
            }

            ((Stack) data).push(ret);
        }
        logger.log(BasicLevel.DEBUG, "End of Visit AdditiveExpression");
        return null;
    }

    public Object visit(ASTUnaryExpression node, Object data) {
        logger.log(BasicLevel.DEBUG, "Visit UnaryExpression");
        visit((SimpleNode) node, data);
        if (((Stack) data).size() > 0) {
            Expression ret = (Expression) ((Stack) data).pop();
            if (node.ops.size() > 0) {
                switch (((Integer) node.ops.get(0)).intValue()) {
                case SpeedoQLConstants.MINUS:
                    ret = new UMinus(ret);
                    break;
                case SpeedoQLConstants.BITWISECOMPL:
                    ret = new Bitwize(ret);
                    break;
                case SpeedoQLConstants.NOT:
                    ret = new Not(ret);
                    break;
                }
            }

            ((Stack) data).push(ret);
        }
        logger.log(BasicLevel.DEBUG, "End of Visit UnaryExpression");
        return null;
    }

    // to be done
    public Object visit(ASTCastExpression node, Object data) {
        ((Stack) data).push(node);
        return null;
    }

    /**
     * 4 cases to manage: (en cours par equipe MEDOR)
     * - Collection.contains(Object o)
     * - Collection.isEmpty()
     * - String.startsWith(String s)
     * - String.endsWith(String s)
     */
    public Object visit(ASTArgumentList node, Object data) {
        logger.log(BasicLevel.DEBUG, "Visit ASTArgumentList");
        int stackInitSize = ((Stack) data).size();

        visit((SimpleNode) node, data);
        if (((Stack) data).size() > 0)
            ((Stack) data).pop();
        logger.log(BasicLevel.DEBUG, "stackInitSize=[" + stackInitSize + "], stackEndSize=[" + ((Stack) data).size() + "]");

        logger.log(BasicLevel.DEBUG, "End of Visit ASTArgumentList");
        return null;
    }

    public Object visit(ASTLiteral node, Object data) {
        if (node.value instanceof Integer) {
            ((Stack) data).push(new BasicOperand(((Integer) node.value).intValue()));
        } else if (node.value instanceof Float) {
            ((Stack) data).push(new BasicOperand(((Float) node.value).floatValue()));
        } else if (node.value instanceof Character) {
            ((Stack) data).push(new BasicOperand(((Character) node.value).charValue()));
        } else if (node.value instanceof String) {
            String s = (String) (node.value);
            s = s.substring(1, s.length()-1);
            ((Stack) data).push(new BasicOperand(s));
        } else if (node.value instanceof Boolean) {
            ((Stack) data).push(new BasicOperand(((Boolean) node.value).booleanValue()));
        }
        // what do I do with a null value ??????????
        //	else ((Stack) data).push(new BasicOperand(null));

        return null;
    }

    public Object visit(ASTType node, Object data) {
        logger.log(BasicLevel.DEBUG, "Visit Type");

        ((Stack) data).push(node);

        logger.log(BasicLevel.DEBUG, "End of Visit Type");
        return null;
    }

    /**
     * qualifiedname could be:
     * - a class field (salary from Employee)
     * - a parameter
     * - a variable
     *
     */
    public Object visit(ASTQualifiedName node, Object data) {
        logger.log(BasicLevel.DEBUG, "sqlqfv: Visit QualifiedName value=[" + node.value + "] stack size=" + ((Stack) data).size());

        String name = (String) (node.value);
        try {
            String prefix = null;
            String[] splitted = splitPath(name);
            if ((params != null) && (params.get(splitted[0]) != null)) {
                logger.log(BasicLevel.DEBUG, "create a parameterOperand with:" + name);

                BasicParameterOperand bpo = (BasicParameterOperand) (params.get(name));
                ((Stack) data).push(bpo);

            } else if ((vars != null) && ((prefix = (String) (vars.get(splitted[0]))) != null)) {
                logger.log(BasicLevel.DEBUG, "create a variableOperand with:" + name);
/*
// TODO a revoir !!!!!!!!!!!!!
BasicVariableOperand bvo = new BasicVariableOperand(PTypeSpace.REFTOP);

((Stack) data).push(bvo);
*/
                Field f = qt.getTupleStructure().getField(name);
                ((Stack) data).push(new BasicFieldOperand(f));

            } else {

                if ("contains".compareTo(splitted[splitted.length - 1]) != 0) {
                    String theName = null;
                    if (name.startsWith("this"))
                        theName = curClass + "." + withoutThis(name);
                    else
                        theName = curClass + "." + name;
                    logger.log(BasicLevel.DEBUG, "create a fieldOperand with:" + theName);

                    Field f = qt.getTupleStructure().getField(theName);

                    ((Stack) data).push(new BasicFieldOperand(f));
                } //else we do NOTHING !!!
            }
        } catch (Exception e) {
            throw new VisitorException(e);
        }
        return null;
    }

}
