/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo;

import junit.framework.TestCase;

import javax.jdo.PersistenceManagerFactory;
import javax.jdo.JDOHelper;

import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.speedo.log.Log;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.jorm.api.PException;
import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.medor.api.MedorException;

import java.util.Properties;

/**
 *
 * @author S.Chassande-Barrioz
 */
public abstract class SpeedoTestHelper extends TestCase {

	public final static String LOG_NAME = "org.objectweb.speedo";

	protected PersistenceManagerFactory pmf = null;
	protected Logger logger = null;


	public SpeedoTestHelper(String s) {
		super(s);
		pmf = JDOHelper.getPersistenceManagerFactory(getPMFProperties());
		logger = Log.getLoggerFactory().getLogger(getLoggerName());
		logger.log(BasicLevel.DEBUG, "PersistenceManagerFactory instanciated (" + pmf.getConnectionURL() + ")");

	}

	protected abstract String getLoggerName();

	public Properties getPMFProperties() {
		String mapperName = System.getProperty("org.objectweb.speedo.mapperName", "rdb");
		String dcn = System.getProperty("javax.jdo.option.DriverClassName", "");
		String user = System.getProperty("javax.jdo.option.ConnectionUserName", "");
		String pass = System.getProperty("javax.jdo.option.ConnectionPassword", "");
		String url = System.getProperty("javax.jdo.option.ConnectionURL");
		if (url == null)
			fail("No database url specified");
		Properties p = new Properties();
		p.setProperty(
			"javax.jdo.PersistenceManagerFactoryClass",
			System.getProperty("javax.jdo.PersistenceManagerFactoryClass",
				"org.objectweb.speedo.Speedo"));
		p.setProperty("org.objectweb.speedo.mapperName", mapperName);
		p.setProperty("javax.jdo.option.DriverClassName", dcn);
		p.setProperty("javax.jdo.option.ConnectionUserName", user);
		p.setProperty("javax.jdo.option.ConnectionPassword", pass);
		p.setProperty("javax.jdo.option.ConnectionURL", url);
		p.setProperty(SpeedoProperties.CACHE_SIZE, "nolimit");
		return p;
	}
}
