/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.basic;

import javax.jdo.PersistenceManager;
import javax.jdo.JDOException;

import org.objectweb.speedo.pobjects.basic.BasicA;
import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestBasicA extends SpeedoTestHelper {

	public TestBasicA(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.basic.TestBasicA";
	}

	public void testCreateInTxAndRemoveInTx2PM() {
		logger.log(BasicLevel.DEBUG, "Create BasicA");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateInTxAndRemoveInTx2PM");
		ba.writeF2(2);
		logger.log(BasicLevel.DEBUG, "Fetch a PersistenceManager");
		PersistenceManager pm = pmf.getPersistenceManager();
		logger.log(BasicLevel.DEBUG, "Start a transaction");
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "Make persistent the instance");
		pm.makePersistent(ba);
		logger.log(BasicLevel.DEBUG, "Fetch the instance identifier");
		Object id = pm.getObjectId(ba);
		logger.log(BasicLevel.DEBUG, "identifier=" + id);
		assertNotNull("Null object identifier", id);
		logger.log(BasicLevel.DEBUG, "Commit the transaction");
		pm.currentTransaction().commit();
		logger.log(BasicLevel.DEBUG, "Close the PersistenceManager");
		pm.close();

		ba = null;

		logger.log(BasicLevel.DEBUG, "Fetch a PersistenceManager");
		pm = pmf.getPersistenceManager();
		logger.log(BasicLevel.DEBUG, "Start a transaction");
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "Fetch the instance by its identifier");
		ba = (BasicA) pm.getObjectById(id, true);
		logger.log(BasicLevel.DEBUG, "instance = " + ba);
		assertNotNull("Object not found", ba);
		assertEquals("Bad f1 value", "testCreateInTxAndRemoveInTx2PM", ba.f1);
		assertEquals("Bad f1 value", 2, ba.f2);

		logger.log(BasicLevel.DEBUG, "Delete the instance");
		pm.deletePersistent(ba);
		logger.log(BasicLevel.DEBUG, "Commit the transaction");
		pm.currentTransaction().commit();
		logger.log(BasicLevel.DEBUG, "Close the PersistenceManager");
		pm.close();
	}

	public void testCreateAndRemoveInTx2PM() {
		logger.log(BasicLevel.DEBUG, "Create BasicA");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateAndRemoveInTx2PM");
		ba.writeF2(2);
		logger.log(BasicLevel.DEBUG, "Fetch a PersistenceManager");
		PersistenceManager pm = pmf.getPersistenceManager();
		logger.log(BasicLevel.DEBUG, "Make persistent the instance");
		pm.makePersistent(ba);
		logger.log(BasicLevel.DEBUG, "Fetch the instance identifier");
		Object id = pm.getObjectId(ba);
		logger.log(BasicLevel.DEBUG, "identifier=" + id);
		assertNotNull("Null object identifier", id);
		logger.log(BasicLevel.DEBUG, "Close the PersistenceManager");
		pm.close();

		ba = null;

		logger.log(BasicLevel.DEBUG, "Fetch a PersistenceManager");
		pm = pmf.getPersistenceManager();
		logger.log(BasicLevel.DEBUG, "Start a transaction");
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "Fetch the instance by its identifier");
		ba = (BasicA) pm.getObjectById(id, true);
		logger.log(BasicLevel.DEBUG, "instance = " + ba);
		assertNotNull("Object not found", ba);
		assertEquals("Bad f1 value", "testCreateAndRemoveInTx2PM", ba.f1);
		assertEquals("Bad f1 value", 2, ba.f2);

		logger.log(BasicLevel.DEBUG, "Delete the instance");
		pm.deletePersistent(ba);
		logger.log(BasicLevel.DEBUG, "Commit the transaction");
		pm.currentTransaction().commit();
		logger.log(BasicLevel.DEBUG, "Close the PersistenceManager");
		pm.close();
	}


	public void testCreateAndRemoveInTx1PM() {
		logger.log(BasicLevel.DEBUG, "Create BasicA");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateAndRemoveInTx1PM");
		ba.writeF2(2);
		logger.log(BasicLevel.DEBUG, "Fetch a PersistenceManager");
		PersistenceManager pm = pmf.getPersistenceManager();
		logger.log(BasicLevel.DEBUG, "Make persistent the instance");
		pm.makePersistent(ba);

		logger.log(BasicLevel.DEBUG, "Start a transaction");
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "Delete the instance");
		pm.deletePersistent(ba);
		logger.log(BasicLevel.DEBUG, "Commit the transaction");
		pm.currentTransaction().commit();
		logger.log(BasicLevel.DEBUG, "Close the PersistenceManager");
		pm.close();
	}

	public void testCreateInTxAndRemoveInTx1PM() {
		logger.log(BasicLevel.DEBUG, "Create BasicA");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateInTxAndRemoveInTx1PM");
		ba.writeF2(2);
		logger.log(BasicLevel.DEBUG, "Fetch a PersistenceManager");
		PersistenceManager pm = pmf.getPersistenceManager();
		logger.log(BasicLevel.DEBUG, "Start a transaction");
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "Make persistent the instance");
		pm.makePersistent(ba);
		logger.log(BasicLevel.DEBUG, "Delete the instance");
		pm.deletePersistent(ba);
		logger.log(BasicLevel.DEBUG, "Commit the transaction");
		try {
			pm.currentTransaction().commit();
		} catch (JDOException e) {
			logger.log(BasicLevel.ERROR, "testCreateAndRemove1PM", ExceptionHelper.getNested(e));
			fail("Jorm does not support export following by an unexport");
		}
		logger.log(BasicLevel.DEBUG, "Close the PersistenceManager");
		pm.close();
	}
}
