package org.objectweb.speedo.runtime.collection;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.api.SpeedoProxy;
import org.objectweb.speedo.pobjects.collection.Employee;
import org.objectweb.util.monolog.api.BasicLevel;

import javax.jdo.PersistenceManager;

public class TestEmployee extends SpeedoTestHelper {

	public TestEmployee(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.collection.TestEmployee";
	}

	public void testCreation1() {
        logger.log(BasicLevel.DEBUG, "Start testCreation");
        logger.log(BasicLevel.DEBUG, "Create Employee");
        Employee e1 = new Employee("e1");
        Employee e2 = new Employee("e2");
        e1.boss = e2;

        PersistenceManager pm = pmf.getPersistenceManager();
        pm.makePersistent(e1);
        Object eId = pm.getObjectId(e1);
        assertNotNull("null object identifier", eId);
        pm.close();
        e1 = null;
        e2 = null;
        pm = pmf.getPersistenceManager();
        e1 = (Employee) pm.getObjectById(eId, true);
        assertNotNull("null instance returned by getObjectById", e1);
        assertEquals("Bad employee name", "e1", e1.name);
        assertNotNull("null instance returned by getObjectById", e1.boss);
        assertEquals("Bad boss name", "e2", e1.boss.name);
        pm.deletePersistent(e1);
        pm.deletePersistent(e1.boss);
        pm.close();
	}

    public void testCreation2() {
        logger.log(BasicLevel.DEBUG, "Start testCreation");
        logger.log(BasicLevel.DEBUG, "Create Employee");
        Employee e1 = new Employee("e1");
        e1.boss = e1;

        PersistenceManager pm = pmf.getPersistenceManager();
        pm.makePersistent(e1);
        Object eId = pm.getObjectId(e1);
        assertNotNull("null object identifier", eId);
        pm.close();

        e1 = null;
        pm = pmf.getPersistenceManager();
        e1 = (Employee) pm.getObjectById(eId, true);
        assertNotNull("null instance returned by getObjectById", e1);
        assertEquals("Bad employee name", "e1", e1.name);
        assertNotNull("null instance returned by getObjectById", e1.boss);
        assertEquals("Bad boss", e1, e1.boss);
        pm.deletePersistent(e1);
        pm.close();
    }

    public void testCreation3() {
        logger.log(BasicLevel.DEBUG, "Start testCreation");
        logger.log(BasicLevel.DEBUG, "Create Employee");
        Employee e1 = new Employee("e1", null);
        Employee e2 = new Employee("e2");

        e1.addFriend(e2);
        assertEquals("Bad friends set size", e1.friends.size(), 1);
        assertTrue("Bad friends set content", e1.friends.contains(e2));

        e1.addInt(1);
        assertEquals("Bad ints set size", e1.ints.size(), 1);
        assertTrue("Bad ints set content", e1.ints.contains(new Integer(1)));

        PersistenceManager pm = pmf.getPersistenceManager();
        pm.makePersistent(e1);
        Object eId = pm.getObjectId(e1);
        assertNotNull("null object identifier", eId);
        pm.close();

        e1 = null;
        pm = pmf.getPersistenceManager();
        e1 = (Employee) pm.getObjectById(eId, true);
        assertNotNull("null instance returned by getObjectById", e1);
        assertEquals("Bad employee name", "e1", e1.name);

        assertNotNull("null collection returned by getObjectById", e1.friends);
        assertEquals("Bad friends set size", e1.friends.size(), 1);
        assertTrue("Bad friends set content", e1.friends.iterator().hasNext());
        assertNotNull("Bad friends set content", e1.friends.iterator().next());
        assertEquals("Bad friends set content", ((Employee) e1.friends.iterator().next()).name, "e2");

        assertNotNull("null collection returned by getObjectById", e1.ints);
        assertEquals("Bad ints set size", e1.ints.size(), 1);
        assertTrue("Bad ints set content", e1.ints.iterator().hasNext());
        assertNotNull("Bad ints set content", e1.ints.iterator().next());
        assertTrue("Bad ints set content", e1.ints.contains(new Integer(1)));
        pm.deletePersistent(e2);
        pm.deletePersistent(e1);
        pm.close();
    }

    public void testCreation4() {
        logger.log(BasicLevel.DEBUG, "Start testCreation");
        logger.log(BasicLevel.DEBUG, "Create Employee");
        Employee e1 = new Employee("e1", null);

        PersistenceManager pm = pmf.getPersistenceManager();
        pm.makePersistent(e1);
        Object eId = pm.getObjectId(e1);
        assertNotNull("null object identifier", eId);

        Employee e2 = new Employee("e2");

        e1.addFriend(e2);
        assertEquals("Bad friends set size", e1.friends.size(), 1);
        assertTrue("Bad friends set content", e1.friends.contains(e2));

        e1.addInt(1);
        assertEquals("Bad ints set size", e1.ints.size(), 1);
        assertTrue("Bad ints set content", e1.ints.contains(new Integer(1)));

        pm.close();

        e1 = null;
        pm = pmf.getPersistenceManager();
        e1 = (Employee) pm.getObjectById(eId, true);
        assertNotNull("null instance returned by getObjectById", e1);
        assertEquals("Bad employee name", "e1", e1.name);

        assertNotNull("null collection returned by getObjectById", e1.friends);
        assertEquals("Bad friends set size", e1.friends.size(), 1);
        assertTrue("Bad friends set content", e1.friends.iterator().hasNext());
        assertNotNull("Bad friends set content", e1.friends.iterator().next());
        assertEquals("Bad friends set content", ((Employee) e1.friends.iterator().next()).name, "e2");

        assertNotNull("null collection returned by getObjectById", e1.ints);
        assertEquals("Bad ints set size", e1.ints.size(), 1);
        assertTrue("Bad ints set content", e1.ints.iterator().hasNext());
        assertNotNull("Bad ints set content", e1.ints.iterator().next());
        assertTrue("Bad ints set content", e1.ints.contains(new Integer(1)));
        Object o = e2.name;

        pm.deletePersistent(e1);
        pm.deletePersistent(e2);
        pm.close();
    }
}
