/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: X. Spengler
 *
 */

package org.objectweb.speedo.runtime.query;

import javax.jdo.Query;
import javax.jdo.PersistenceManager;
import javax.jdo.JDOException;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.pobjects.ref.Employee;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Collection;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

/**
 * This test tests the access to 2 fields from the Employee class (name
 * and salary), Salary is a Float (with 2 parameters, a String and a Float
 * object).
 */
public class TestQueries extends SpeedoTestHelper {

	public TestQueries(String name) {
		super(name);
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.query.parameter";
	}

    public void test0Parameter() {
        logger.log(BasicLevel.DEBUG, "test0Parameter");
        PersistenceManager pm = pmf.getPersistenceManager();
        try {
            Class empClass = Employee.class;
            Query query = pm.newQuery(empClass);
            Collection col = (Collection) query.execute();
            Iterator it = col.iterator();
            while(it.hasNext()) {
                Employee e = (Employee) it.next();
                assertNotNull("null element in the query result", e);
                assertNotNull("null name  of a element in the query result", e.name);
            }
            query.close(col);
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
    }

    public void test10rder() {
        logger.log(BasicLevel.DEBUG, "test10rder");
        PersistenceManager pm = pmf.getPersistenceManager();
        try {
            Class empClass = Employee.class;
            Query query = pm.newQuery(empClass);
            query.setOrdering("name ascending");
            Collection col = (Collection) query.execute();
            Iterator it = col.iterator();
            for(int i=0; i<POBuilder.nameOrder.length; i++) {
                assertTrue("No enough values in the result", it.hasNext());
                Employee e = (Employee) it.next();
                assertNotNull("null element in the query result", e);
                assertEquals("Bad order (" + i + ")",
                    POBuilder.names[POBuilder.nameOrder[i]],e.name);
            }
            query.close(col);
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
    }

    public void test20rder() {
        logger.log(BasicLevel.DEBUG, "test20rder");
        PersistenceManager pm = pmf.getPersistenceManager();
        try {
            Class empClass = Employee.class;
            Query query = pm.newQuery(empClass);
            query.setOrdering("salary descending, name ascending");
            Collection col = (Collection) query.execute();
            Iterator it = col.iterator();
            for(int i=0; i<POBuilder.nameOrder.length; i++) {
                assertTrue("No enough values in the result", it.hasNext());
                Employee e = (Employee) it.next();
                assertNotNull("null element in the query result", e);
                assertEquals("Bad order (" + i + ")",
                    POBuilder.names[POBuilder.salariesOrder[i]],e.name);
            }
            query.close(col);
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
    }

    public void test1Parameter() {
        logger.log(BasicLevel.DEBUG, "test1Parameter");
        PersistenceManager pm = pmf.getPersistenceManager();
        try {
            Class empClass = Employee.class;
            Query query = pm.newQuery(empClass);
            query.declareParameters("String aName");
            query.setFilter("(name == aName)");
            Collection col = (Collection) query.execute(POBuilder.names[1]);
            Iterator iter = col.iterator();
            assertTrue("The query result is empty", iter.hasNext());
            Employee e = (Employee) iter.next();
            assertNotNull("Null collection element", e);
            assertEquals("Bad employee name", POBuilder.names[1], e.name);
            assertTrue("More than one element in the query result", !iter.hasNext());
            assertEquals("Bad query result size", 1, col.size());
            query.close(col);
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
    }

    public void testFieldParameter() {
        logger.log(BasicLevel.DEBUG, "testFieldParameter");
        PersistenceManager pm = pmf.getPersistenceManager();
        try {
            Class empClass = Employee.class;
            Query query = pm.newQuery(empClass);

            String filterFieldParameter = "((salary == mysalary) && (name == myname))";
            String param = "String myname, Float mysalary";

            query.declareParameters(param);
            query.setFilter(filterFieldParameter);

            Collection col = (Collection)
                    query.execute(POBuilder.names[1], new Float(3000.0));
            Iterator iter = col.iterator();
            assertTrue("The query result is empty", iter.hasNext());
            Employee e = (Employee) iter.next();
            assertNotNull("Null collection element", e);
            assertEquals("Bad employee name", POBuilder.names[1], e.name);
            assertEquals("Bad employee salary", new Float(3000.0), new Float(e.salary));
            assertTrue("More than one element in the query result", !iter.hasNext());
            assertEquals("Bad query result size", 1, col.size());
            query.close(col);
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
    }

	public void test2Parameters() {
        logger.log(BasicLevel.DEBUG, "test2Parameters");
        PersistenceManager pm = pmf.getPersistenceManager();
		try {
			Class empClass = Employee.class;
			Query query = pm.newQuery(empClass);
			query.declareParameters("String aName, String anotherName");
			query.setFilter("((name == aName) || (name == anotherName))");
			Collection col = (Collection)
                    query.execute(POBuilder.names[1], POBuilder.names[2]);
            Iterator iter = col.iterator();
            assertTrue("The query result is empty", iter.hasNext());
            Employee e1 = (Employee) iter.next();
            assertNotNull("Null collection element 1", e1);
            assertTrue("Only one element in the query result", iter.hasNext());
            Employee e2 = (Employee) iter.next();
            assertNotNull("Null collection element 2", e2);
            assertTrue("More than one element in the query result", !iter.hasNext());
            assertEquals("Bad query result size", 2, col.size());
            query.close(col);

            if (POBuilder.names[1].equals(e1.name)) {
                assertEquals("Bad employee name", POBuilder.names[2], e2.name);
            } else if (POBuilder.names[1].equals(e2.name)) {
                assertEquals("Bad employee name", POBuilder.names[2], e1.name);
            } else {
                fail("Bad employee name");
            }
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
	}

	public void test3Parameters() {
        logger.log(BasicLevel.DEBUG, "test3Parameters");
        PersistenceManager pm = pmf.getPersistenceManager();
		try {
			Class empClass = Employee.class;
			Query query = pm.newQuery(empClass);
			String filter = "((name == aName) | (name == anotherName) | (name == thirdName))";
			String param = "String aName, String anotherName, String thirdName";
			query.declareParameters(param);
			query.setFilter(filter);
			Collection col = (Collection) query.execute(POBuilder.names[1],
														POBuilder.names[2],
														POBuilder.names[3]);
            Iterator iter = col.iterator();
            Collection expectedNames = new ArrayList(3);
            Collection foundNames = new ArrayList(3);
            expectedNames.add(POBuilder.names[1]);
            expectedNames.add(POBuilder.names[2]);
            expectedNames.add(POBuilder.names[3]);
            assertTrue("The query result is empty", iter.hasNext());
            Employee e = (Employee) iter.next();
            assertNotNull("Null collection element 1", e);
            foundNames.add(e.name);
            assertTrue("Only one element in the query result", iter.hasNext());
            e = (Employee) iter.next();
            assertNotNull("Null collection element 2", e);
            foundNames.add(e.name);
            e = (Employee) iter.next();
            assertNotNull("Null collection element 3", e);
            foundNames.add(e.name);
            assertTrue("More than one element in the query result", !iter.hasNext());
            assertEquals("Bad query result size", expectedNames.size(), col.size());
            query.close(col);

            assertTrue("Bad Employee names, expected:"
                    + expectedNames + " / found: " + foundNames,
                    expectedNames.containsAll(foundNames)
                    && foundNames.containsAll(expectedNames));
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
 	}

	public void testMapParameters() {
        logger.log(BasicLevel.DEBUG, "testMapParameters");
        PersistenceManager pm = pmf.getPersistenceManager();
		try {
			Class empClass = Employee.class;
			Query query = pm.newQuery(empClass);

			String filter = "((name == aName) || (name == anotherName))";
			String param = "String aName, String anotherName";

			query.declareParameters(param);
			query.setFilter(filter);

			Map map = new HashMap();
			map.put("aName", POBuilder.names[1]);
			map.put("anotherName", POBuilder.names[2]);

			Collection col = (Collection) query.executeWithMap(map);
            Iterator iter = col.iterator();
            Collection foundNames = new ArrayList(2);
            Collection expectedNames = map.values();

            assertTrue("The query result is empty", iter.hasNext());
            Employee e = (Employee) iter.next();
            assertNotNull("Null collection element 1", e);
            foundNames.add(e.name);
            assertTrue("Only one element in the query result", iter.hasNext());
            e = (Employee) iter.next();
            assertNotNull("Null collection element 2", e);
            foundNames.add(e.name);
            assertTrue("More than one element in the query result", !iter.hasNext());
            assertEquals("Bad query result size", expectedNames.size(), col.size());
            query.close(col);

            assertTrue("Bad Employee names, expected:"
                    + expectedNames + " / found: " + foundNames,
                    expectedNames.containsAll(foundNames)
                    && foundNames.containsAll(expectedNames));
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
	}

	public void testArrayParameters() {
        logger.log(BasicLevel.DEBUG, "testArrayParameters");
        PersistenceManager pm = pmf.getPersistenceManager();
		try {
			Class empClass = Employee.class;
			Query query = pm.newQuery(empClass);

			String filter = "((name == aName) | (name == anotherName))";
			String param = "String aName, String anotherName";

			query.declareParameters(param);
			query.setFilter(filter);

            Collection expectedNames = new ArrayList(2);
            expectedNames.add(POBuilder.names[1]);
            expectedNames.add(POBuilder.names[2]);
			Collection col = (Collection) query.executeWithArray(expectedNames.toArray());
            Iterator iter = col.iterator();

            Collection foundNames = new ArrayList(2);
            assertTrue("The query result is empty", iter.hasNext());
            Employee e = (Employee) iter.next();
            assertNotNull("Null collection element 1", e);
            foundNames.add(e.name);
            assertTrue("Only one element in the query result", iter.hasNext());
            e = (Employee) iter.next();
            assertNotNull("Null collection element 2", e);
            foundNames.add(e.name);
            assertTrue("More than one element in the query result", !iter.hasNext());
            assertEquals("Bad query result size", expectedNames.size(), col.size());
            query.close(col);

            assertTrue("Bad Employee names, expected:"
                    + expectedNames + " / found: " + foundNames,
                    expectedNames.containsAll(foundNames)
                    && foundNames.containsAll(expectedNames));
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
	}

    public void testFieldRef() {
        logger.log(BasicLevel.DEBUG, "testFieldRef");
        PersistenceManager pm = pmf.getPersistenceManager();
        try {
            Class empClass = Employee.class;
            Query query = pm.newQuery(empClass);
            String filterFieldRef = "((dept.name == depName) && (name == \""
                + POBuilder.names[0] + "\"))";
            String param = "String depName";

            query.declareParameters(param);
            query.setFilter(filterFieldRef);

            Collection col = (Collection) query.execute("RD");
            Iterator iter = col.iterator();
            assertTrue("The query result is empty", iter.hasNext());
            Employee e = (Employee) iter.next();
            assertNotNull("Null collection element", e);
            assertEquals("Bad employee name", POBuilder.names[0], e.name);
            assertTrue("More than one element in the query result", !iter.hasNext());
            assertEquals("Bad query result size", 1, col.size());
            query.close(col);
        } catch (Exception e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
    }

	public void testThis() {
        logger.log(BasicLevel.DEBUG, "testThis");
        PersistenceManager pm = pmf.getPersistenceManager();
		try {
			Class empClass = Employee.class;
			Query query = pm.newQuery(empClass);

			String filter = "(this.name == aName)";
			String param = "String aName";

			query.declareParameters(param);
			query.setFilter(filter);

			Collection col = (Collection) query.execute(POBuilder.names[1]);
            Iterator iter = col.iterator();
            assertTrue("The query result is empty", iter.hasNext());
            Employee e = (Employee) iter.next();
            assertNotNull("Null collection element", e);
            assertEquals("Bad employee name", POBuilder.names[1], e.name);
            assertTrue("More than one element in the query result", !iter.hasNext());
            assertEquals("Bad query result size", 1, col.size());
            query.close(col);
		} catch (Exception e) {
			Exception ie = ExceptionHelper.getNested(e);
			logger.log(BasicLevel.ERROR, "", ie);
			fail(ie.getMessage());
		} finally {
            pm.close();
        }
	}
}
