/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.userid;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.pobjects.userid.BasicB;
import org.objectweb.speedo.pobjects.userid.StringLongId;
import org.objectweb.speedo.pobjects.userid.StringUserId;
import org.objectweb.speedo.pobjects.userid.IntUserId;
import org.objectweb.util.monolog.api.BasicLevel;

import javax.jdo.PersistenceManager;
import javax.jdo.JDOException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.ObjectOutputStream;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.File;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestUserId extends SpeedoTestHelper {

    private final static String OID_FILE_NAME = "TestUserId_test1.id";

	public TestUserId(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + "rt.userid";
	}

    public void test1MakePersistent() {
        try {
            PersistenceManager pm = pmf.getPersistenceManager();
            BasicB bb = new BasicB();
            bb.id1 = "azerty";
            bb.id2 = 217;
            bb.f1 = "blabla";
            bb.f2 = 456;
            pm.makePersistent(bb);
            StringLongId id = (StringLongId) pm.getObjectId(bb);
            logger.log(BasicLevel.DEBUG, "UserId=" + id);
            pm.close();
            assertNotNull("Null Object id", id);
            ObjectOutputStream os = new ObjectOutputStream(
                    new FileOutputStream(OID_FILE_NAME));
            os.writeObject(id);
            os.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void test2LoadNDelete() {
        try {
            PersistenceManager pm = pmf.getPersistenceManager();
            ObjectInputStream is = new ObjectInputStream(
                    new FileInputStream(OID_FILE_NAME));
            StringLongId id = (StringLongId) is.readObject();
            is.close();
            new File(OID_FILE_NAME).delete();
            BasicB bb = (BasicB) pm.getObjectById(id, true);
            assertNotNull("Null Object", bb);
            pm.deletePersistent(bb);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void testUserId() {
        try {
            String id1 = "querty";
            int id2 = 12000;
            String f1 = "blabla";
            int f2 = 456;

            PersistenceManager pm = pmf.getPersistenceManager();
            BasicB bb = new BasicB();
            bb.id1 = id1;
            bb.id2 = id2;
            bb.f1 = f1;
            bb.f2 = f2;
            pm.makePersistent(bb);
            StringLongId id = (StringLongId) pm.getObjectId(bb);
            logger.log(BasicLevel.DEBUG, "UserId=" + id);
            assertNotNull("identifier is null", id);
            String strid = id.toString();
            bb = null;
            pm.close();

            pm = pmf.getPersistenceManager();
            id = new StringLongId(id1, id2);
            bb = (BasicB) pm.getObjectById(id, false);
            assertNotNull("returned object is null", bb);
            assertEquals("Bad field 'id1' value", id1, bb.id1);
            assertEquals("Bad field 'di2' value", id2, bb.id2);
            assertEquals("Bad field 'f1' value", f1, bb.f1);
            assertEquals("Bad field 'f1' value", f2, bb.f2);
            pm.deletePersistent(bb);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void testStringId() {
        try {
            String name = "testStringId";
            int f1 = 12;
            Object id;
            String strid;
            PersistenceManager pm = pmf.getPersistenceManager();
            StringUserId sui = new StringUserId(name, f1);
            pm.makePersistent(sui);
            id = pm.getObjectId(sui);
            assertNotNull("identifier is null", id);
            strid = id.toString();
            id = null;
            pm.close();
            sui = null; // garbage object
            pm = pmf.getPersistenceManager();
            id = pm.newObjectIdInstance(IntUserId.class, strid);
            sui = (StringUserId) pm.getObjectById(id, false);
            assertNotNull("returned object is null", sui);
            assertEquals("Bad field 'name' value", name, sui.getName());
            assertEquals("Bad field 'f1' value", f1, sui.getF1());
            pm.deletePersistent(sui);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void testIntId() {
        try {
            String f1 = "testInt";
            int name = 13;
            Object id;
            PersistenceManager pm = pmf.getPersistenceManager();
            IntUserId iui = new IntUserId(name, f1);
            pm.makePersistent(iui);
            id = pm.getObjectId(iui);
            assertNotNull("identifier is null", id);
            pm.close();
            iui = null; // garbage object
            pm = pmf.getPersistenceManager();
            id = pm.newObjectIdInstance(IntUserId.class, id.toString());
            assertNotNull("built identifier is null", id);
            iui = (IntUserId) pm.getObjectById(id, false);
            assertNotNull("returned object is null", iui);
            assertEquals("Bad field 'name' value", name, iui.getName());
            assertEquals("Bad field 'f1' value", f1, iui.getF1());
            pm.deletePersistent(iui);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }
}
